searchData={"items":[{"type":"module","title":"Cachex","doc":"Cachex provides a straightforward interface for in-memory key/value storage.\n\nCachex is an extremely fast, designed for caching but also allowing for more\ngeneral in-memory storage. The main goal of Cachex is achieve a caching\nimplementation with a wide array of options, without sacrificing performance.\nInternally, Cachex is backed by ETS, allowing for an easy-to-use interface\nsitting upon extremely well tested tools.\n\nCachex comes with support for all of the following (amongst other things):\n\n- Time-based key expirations\n- Maximum size protection\n- Pre/post execution hooks\n- Proactive/reactive cache warming\n- Transactions and row locking\n- Asynchronous write operations\n- Distribution across app nodes\n- Syncing to a local filesystem\n- Idiomatic cache streaming\n- Batched write operations\n- User command invocation\n- Statistics gathering\n\nAll features are optional to allow you to tune based on the throughput needed.\n\nPlease see `Cachex.start_link/2` inside the Cachex [documentation](https://hexdocs.pm/cachex)\nfor further details about how to configure these  options and example usage.","ref":"Cachex.html"},{"type":"function","title":"Cachex.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Cachex.html#child_spec/1"},{"type":"function","title":"Cachex.clear/2","doc":"Removes all entries from a cache.\n\nThe returned numeric value will contain the total number of keys removed\nfrom the cache. This is equivalent to running `size/2` before running\nthe internal clear operation.","ref":"Cachex.html#clear/2"},{"type":"function","title":"Examples - Cachex.clear/2","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.get(:my_cache, \"key\")\n    iex> Cachex.size(:my_cache)\n    { :ok, 1 }\n\n    iex> Cachex.clear(:my_cache)\n    { :ok, 1 }\n\n    iex> Cachex.size(:my_cache)\n    { :ok, 0 }","ref":"Cachex.html#clear/2-examples"},{"type":"function","title":"Cachex.decr/4","doc":"Decrements an entry in the cache.\n\nThis will overwrite any value that was previously set against the provided key.","ref":"Cachex.html#decr/4"},{"type":"function","title":"Options - Cachex.decr/4","doc":"* `:default`\n\n    An initial value to set the key to if it does not exist. This will\n    take place *before* the decrement call. Defaults to 0.","ref":"Cachex.html#decr/4-options"},{"type":"function","title":"Examples - Cachex.decr/4","doc":"iex> Cachex.put(:my_cache, \"my_key\", 10)\n    iex> Cachex.decr(:my_cache, \"my_key\")\n    { :ok, 9 }\n\n    iex> Cachex.put(:my_cache, \"my_new_key\", 10)\n    iex> Cachex.decr(:my_cache, \"my_new_key\", 5)\n    { :ok, 5 }\n\n    iex> Cachex.decr(:my_cache, \"missing_key\", 5, default: 2)\n    { :ok, -3 }","ref":"Cachex.html#decr/4-examples"},{"type":"function","title":"Cachex.del/3","doc":"Removes an entry from a cache.\n\nThis will return `{ :ok, true }` regardless of whether a key has been removed\nor not. The `true` value can be thought of as \"is key no longer present?\".","ref":"Cachex.html#del/3"},{"type":"function","title":"Examples - Cachex.del/3","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.get(:my_cache, \"key\")\n    { :ok, \"value\" }\n\n    iex> Cachex.del(:my_cache, \"key\")\n    { :ok, true }\n\n    iex> Cachex.get(:my_cache, \"key\")\n    { :ok, nil }","ref":"Cachex.html#del/3-examples"},{"type":"function","title":"Cachex.empty?/2","doc":"Determines whether a cache contains any entries.\n\nThis does not take the expiration time of keys into account. As such,\nif there are any unremoved (but expired) entries in the cache, they\nwill be included in the returned determination.","ref":"Cachex.html#empty?/2"},{"type":"function","title":"Examples - Cachex.empty?/2","doc":"iex> Cachex.put(:my_cache, \"key1\", \"value1\")\n    iex> Cachex.empty?(:my_cache)\n    { :ok, false }\n\n    iex> Cachex.clear(:my_cache)\n    iex> Cachex.empty?(:my_cache)\n    { :ok, true }","ref":"Cachex.html#empty?/2-examples"},{"type":"function","title":"Cachex.execute/3","doc":"Executes multiple functions in the context of a cache.\n\nThis can be used when carrying out several cache operations at once\nto avoid the overhead of cache loading and jumps between processes.\n\nThis does not provide a transactional execution, it simply avoids\nthe overhead involved in the initial calls to a cache. For a transactional\nimplementation, please see `transaction/3`.\n\nTo take advantage of the cache context, ensure to use the cache\ninstance provided when executing cache calls. If this is not done\nyou will see zero benefits from using `execute/3`.","ref":"Cachex.html#execute/3"},{"type":"function","title":"Examples - Cachex.execute/3","doc":"iex> Cachex.put(:my_cache, \"key1\", \"value1\")\n    iex> Cachex.put(:my_cache, \"key2\", \"value2\")\n    iex> Cachex.execute(:my_cache, fn(worker) ->\n    ...>   val1 = Cachex.get!(worker, \"key1\")\n    ...>   val2 = Cachex.get!(worker, \"key2\")\n    ...>   [val1, val2]\n    ...> end)\n    { :ok, [ \"value1\", \"value2\" ] }","ref":"Cachex.html#execute/3-examples"},{"type":"function","title":"Cachex.exists?/3","doc":"Determines whether an entry exists in a cache.\n\nThis will take expiration times into account, meaning that\nexpired entries will not be considered to exist.","ref":"Cachex.html#exists?/3"},{"type":"function","title":"Examples - Cachex.exists?/3","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.exists?(:my_cache, \"key\")\n    { :ok, true }\n\n    iex> Cachex.exists?(:my_cache, \"missing_key\")\n    { :ok, false }","ref":"Cachex.html#exists?/3-examples"},{"type":"function","title":"Cachex.expire/4","doc":"Places an expiration time on an entry in a cache.\n\nThe provided expiration must be a integer value representing the\nlifetime of the entry in milliseconds. If the provided value is\nnot positive, the entry will be immediately evicted.\n\nIf the entry does not exist, no changes will be made in the cache.","ref":"Cachex.html#expire/4"},{"type":"function","title":"Examples - Cachex.expire/4","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.expire(:my_cache, \"key\", :timer.seconds(5))\n    { :ok, true }\n\n    iex> Cachex.expire(:my_cache, \"missing_key\", :timer.seconds(5))\n    { :ok, false }","ref":"Cachex.html#expire/4-examples"},{"type":"function","title":"Cachex.expire_at/4","doc":"Updates an entry in a cache to expire at a given time.\n\nUnlike `expire/4` this call uses an instant in time, rather than a\nduration. The same semantics apply as calls to `expire/4` in that\ninstants which have passed will result in immediate eviction.","ref":"Cachex.html#expire_at/4"},{"type":"function","title":"Examples - Cachex.expire_at/4","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.expire_at(:my_cache, \"key\", 1455728085502)\n    { :ok, true }\n\n    iex> Cachex.expire_at(:my_cache, \"missing_key\", 1455728085502)\n    { :ok, false }","ref":"Cachex.html#expire_at/4-examples"},{"type":"function","title":"Cachex.export/2","doc":"Exports all entries from a cache.\n\nThis provides a raw read of the entire backing table into a list\nof cache records for export purposes.\n\nThis function is very heavy, so it should typically only be used\nwhen debugging and/or exporting of tables (although the latter case\nshould really use `Cachex.save/3`).","ref":"Cachex.html#export/2"},{"type":"function","title":"Examples - Cachex.export/2","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.export(:my_cache)\n    { :ok, [ { :entry, \"key\", 1538714590095, nil, \"value\" } ] }","ref":"Cachex.html#export/2-examples"},{"type":"function","title":"Cachex.fetch/4","doc":"Fetches an entry from a cache, generating a value on cache miss.\n\nIf the entry requested is found in the cache, this function will\noperate in the same way as `get/3`. If the entry is not contained\nin the cache, the provided fallback function will be executed.\n\nA fallback function is a function used to lazily generate a value\nto place inside a cache on miss. Consider it a way to achieve the\nability to create a read-through cache.\n\nA fallback function should return a Tuple consisting of a `:commit`\nor `:ignore` tag and a value. If the Tuple is tagged `:commit` the\nvalue will be placed into the cache and then returned. If tagged\n`:ignore` the value will be returned without being written to the\ncache. If you return a value which does not fit this structure, it\nwill be assumed that you are committing the value.\n\nAs of Cachex v3.6, you can also provide a third element in a `:commit`\nTuple, to allow passthrough of options from within your fallback. The\noptions supported in this list match the options you can provide to a\ncall of `put/4`. An example is the `:ttl` option to set an expiration\nfrom directly inside your fallback.\n\nIf a fallback function has an arity of 1, the requested entry key\nwill be passed through to allow for contextual computation. If a\nfunction has an arity of 2, the `:provide` option from the global\n`:fallback` cache option will be provided as the second argument.\nThis is to allow easy state sharing, such as remote clients. If a\nfunction has an arity of 0, it will be executed without arguments.\n\nIf a cache has been initialized with a default fallback function\nin the `:fallback` option at cache startup, the third argument to\nthis call becomes optional.","ref":"Cachex.html#fetch/4"},{"type":"function","title":"Examples - Cachex.fetch/4","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.fetch(:my_cache, \"key\", fn(key) ->\n    ...>   { :commit, String.reverse(key) }\n    ...> end)\n    { :ok, \"value\" }\n\n    iex> Cachex.fetch(:my_cache, \"missing_key\", fn(key) ->\n    ...>   { :ignore, String.reverse(key) }\n    ...> end)\n    { :ignore, \"yek_gnissim\" }\n\n    iex> Cachex.fetch(:my_cache, \"missing_key\", fn(key) ->\n    ...>   { :commit, String.reverse(key) }\n    ...> end)\n    { :commit, \"yek_gnissim\" }\n\n    iex> Cachex.fetch(:my_cache, \"missing_key_expires\", fn(key) ->\n    ...>   { :commit, String.reverse(key), expire: :timer.seconds(60) }\n    ...> end)\n    { :commit, \"seripxe_yek_gnissim\", [expire: 60000] }","ref":"Cachex.html#fetch/4-examples"},{"type":"function","title":"Cachex.get/3","doc":"Retrieves an entry from a cache.","ref":"Cachex.html#get/3"},{"type":"function","title":"Examples - Cachex.get/3","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.get(:my_cache, \"key\")\n    { :ok, \"value\" }\n\n    iex> Cachex.get(:my_cache, \"missing_key\")\n    { :ok, nil }","ref":"Cachex.html#get/3-examples"},{"type":"function","title":"Cachex.get_and_update/4","doc":"Retrieves and updates an entry in a cache.\n\nThis operation can be seen as an internal mutation, meaning that any previously\nset expiration time is kept as-is.\n\nThis function accepts the same return syntax as fallback functions, in that if\nyou return a Tuple of the form `{ :ignore, value }`, the value is returned from\nthe call but is not written to the cache. You can use this to abandon writes\nwhich began eagerly (for example if a key is actually missing)\n\nSee the `fetch/4` documentation for more information on return formats.","ref":"Cachex.html#get_and_update/4"},{"type":"function","title":"Examples - Cachex.get_and_update/4","doc":"iex> Cachex.put(:my_cache, \"key\", [2])\n    iex> Cachex.get_and_update(:my_cache, \"key\", &([1|&1]))\n    { :commit, [1, 2] }\n\n    iex> Cachex.get_and_update(:my_cache, \"missing_key\", fn\n    ...>   (nil) -> { :ignore, nil }\n    ...>   (val) -> { :commit, [ \"value\" | val ] }\n    ...> end)\n    { :ignore, nil }","ref":"Cachex.html#get_and_update/4-examples"},{"type":"function","title":"Cachex.import/3","doc":"Imports an export set into a cache.\n\nThis provides a raw import of a previously exported cache via the use\nof the `export/2` command.","ref":"Cachex.html#import/3"},{"type":"function","title":"Examples - Cachex.import/3","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.import(:my_cache, [ { :entry, \"key\", \"value\", 1538714590095, nil } ])\n    { :ok, 1 }","ref":"Cachex.html#import/3-examples"},{"type":"function","title":"Cachex.incr/4","doc":"Increments an entry in the cache.\n\nThis will overwrite any value that was previously set against the provided key.","ref":"Cachex.html#incr/4"},{"type":"function","title":"Options - Cachex.incr/4","doc":"* `:default`\n\n    An initial value to set the key to if it does not exist. This will\n    take place *before* the increment call. Defaults to 0.","ref":"Cachex.html#incr/4-options"},{"type":"function","title":"Examples - Cachex.incr/4","doc":"iex> Cachex.put(:my_cache, \"my_key\", 10)\n    iex> Cachex.incr(:my_cache, \"my_key\")\n    { :ok, 11 }\n\n    iex> Cachex.put(:my_cache, \"my_new_key\", 10)\n    iex> Cachex.incr(:my_cache, \"my_new_key\", 5)\n    { :ok, 15 }\n\n    iex> Cachex.incr(:my_cache, \"missing_key\", 5, default: 2)\n    { :ok, 7 }","ref":"Cachex.html#incr/4-examples"},{"type":"function","title":"Cachex.inspect/3","doc":"Inspects various aspects of a cache.\n\nThese operations should be regarded as debug tools, and should really\nonly happen outside of production code (unless absolutely) necessary.\n\nAccepted options are only provided for convenience and should not be\nheavily relied upon.  They are not part of the public interface\n(despite being documented) and as such  may be removed at any time\n(however this does not mean that they will be).\n\nPlease use cautiously. `inspect/2` is provided mainly for testing\npurposes and so performance isn't as much of a concern. It should\nalso be noted that `inspect/2` will *always* operate locally.","ref":"Cachex.html#inspect/3"},{"type":"function","title":"Options - Cachex.inspect/3","doc":"* `:cache`\n\n    Retrieves the internal cache record for a cache.\n\n  * `{ :entry, key }`\n\n    Retrieves a raw entry record from inside a cache.\n\n  * `{ :expired, :count }`\n\n    Retrieves the number of expired entries which currently live in the cache\n    but have not yet been removed by cleanup tasks (either scheduled or lazy).\n\n  * `{ :expired, :keys }`\n\n    Retrieves the list of expired entry keys which current live in the cache\n    but have not yet been removed by cleanup tasks (either scheduled or lazy).\n\n  * `{ :janitor, :last }`\n\n    Retrieves metadata about the last execution of the Janitor service for\n    the specified cache.\n\n  * `{ :memory, :bytes }`\n\n    Retrieves an approximate memory footprint of a cache in bytes.\n\n  * `{ :memory, :binary }`\n\n    Retrieves an approximate memory footprint of a cache in binary format.\n\n  * `{ :memory, :words }`\n\n    Retrieve an approximate memory footprint of a cache as a number of\n    machine words.","ref":"Cachex.html#inspect/3-options"},{"type":"function","title":"Examples - Cachex.inspect/3","doc":"iex> Cachex.inspect(:my_cache, :cache)\n    {:ok,\n      {:cache, :my_cache, %{}, {:expiration, nil, 3000, true}, {:fallback, nil, nil},\n        {:hooks, [], []}, {:limit, nil, Cachex.Policy.LRW, 0.1, []}, false, []}}\n\n    iex> Cachex.inspect(:my_cache, { :entry, \"my_key\" } )\n    { :ok, { :entry, \"my_key\", 1475476615662, 1, \"my_value\" } }\n\n    iex> Cachex.inspect(:my_cache, { :expired, :count })\n    { :ok, 0 }\n\n    iex> Cachex.inspect(:my_cache, { :expired, :keys })\n    { :ok, [ ] }\n\n    iex> Cachex.inspect(:my_cache, { :janitor, :last })\n    { :ok, %{ count: 0, duration: 57, started: 1475476530925 } }\n\n    iex> Cachex.inspect(:my_cache, { :memory, :binary })\n    { :ok, \"10.38 KiB\" }\n\n    iex> Cachex.inspect(:my_cache, { :memory, :bytes })\n    { :ok, 10624 }\n\n    iex> Cachex.inspect(:my_cache, { :memory, :words })\n    { :ok, 1328 }","ref":"Cachex.html#inspect/3-examples"},{"type":"function","title":"Cachex.invoke/4","doc":"Invokes a custom command against a cache entry.\n\nThe provided command name must be a valid command which was\npreviously attached to the cache in calls to `start_link/2`.","ref":"Cachex.html#invoke/4"},{"type":"function","title":"Examples - Cachex.invoke/4","doc":"iex> import Cachex.Spec\n    iex>\n    iex> Cachex.start_link(:my_cache, [\n    ...>    commands: [\n    ...>      last: command(type: :read, execute: &List.last/1)\n    ...>    ]\n    ...> ])\n    { :ok, _pid }\n\n    iex> Cachex.put(:my_cache, \"my_list\", [ 1, 2, 3 ])\n    iex> Cachex.invoke(:my_cache, :last, \"my_list\")\n    { :ok, 3 }","ref":"Cachex.html#invoke/4-examples"},{"type":"function","title":"Cachex.keys/2","doc":"Retrieves a list of all entry keys from a cache.\n\nThe order these keys are returned should be regarded as unordered.","ref":"Cachex.html#keys/2"},{"type":"function","title":"Examples - Cachex.keys/2","doc":"iex> Cachex.put(:my_cache, \"key1\", \"value1\")\n    iex> Cachex.put(:my_cache, \"key2\", \"value2\")\n    iex> Cachex.put(:my_cache, \"key3\", \"value3\")\n    iex> Cachex.keys(:my_cache)\n    { :ok, [ \"key2\", \"key1\", \"key3\" ] }\n\n    iex> Cachex.clear(:my_cache)\n    iex> Cachex.keys(:my_cache)\n    { :ok, [] }","ref":"Cachex.html#keys/2-examples"},{"type":"function","title":"Cachex.persist/3","doc":"Removes an expiration time from an entry in a cache.","ref":"Cachex.html#persist/3"},{"type":"function","title":"Examples - Cachex.persist/3","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\", expiration: 1000)\n    iex> Cachex.persist(:my_cache, \"key\")\n    { :ok, true }\n\n    iex> Cachex.persist(:my_cache, \"missing_key\")\n    { :ok, false }","ref":"Cachex.html#persist/3-examples"},{"type":"function","title":"Cachex.prune/3","doc":"Prunes to a maximum size of records in a cache.\n\nPruning is done via a Least Recently Written (LRW) approach, determined by the\nmodification time inside each cache record to avoid storing additional state.\n\nFor full details on this feature, please see the section of the documentation\nrelated to limitation of caches.","ref":"Cachex.html#prune/3"},{"type":"function","title":"Options - Cachex.prune/3","doc":"* `:buffer`\n\n    Allows customization of the internal batching when paginating the cursor\n    coming back from ETS. It's unlikely this will ever need changing.\n\n  * `:reclaim`\n\n    Provides control over thrashing by evicting an additonal number of cache\n    entries beyond the maximum size. This option accepts a percentage (as a\n    decimal) of extra keys to evict, to provide buffer between pruning passes.\n    Defaults to `0.1` (i.e. 10%).","ref":"Cachex.html#prune/3-options"},{"type":"function","title":"Examples - Cachex.prune/3","doc":"iex> Cachex.put(:my_cache, \"key1\", \"value1\")\n    { :ok, true }\n\n    iex> :timer.sleep(1)\n    :ok\n\n    iex> Cachex.put(:my_cache, \"key2\", \"value2\")\n    { :ok, true }\n\n    iex> Cachex.prune(:my_cache, 1, reclaim: 0)\n    { :ok, true }\n\n    iex> Cachex.keys(:my_cache)\n    { :ok, [ \"key2\"] }","ref":"Cachex.html#prune/3-examples"},{"type":"function","title":"Cachex.purge/2","doc":"Triggers a cleanup of all expired entries in a cache.\n\nThis can be used to implement custom eviction policies rather than\nrelying on the internal Janitor service. Take care when using this\nmethod though; calling `purge/2` manually will result in a purge\nfiring inside the calling process.","ref":"Cachex.html#purge/2"},{"type":"function","title":"Examples - Cachex.purge/2","doc":"iex> Cachex.purge(:my_cache)\n    { :ok, 15 }","ref":"Cachex.html#purge/2-examples"},{"type":"function","title":"Cachex.put/4","doc":"Places an entry in a cache.\n\nThis will overwrite any value that was previously set against the provided key,\nand overwrite any TTLs which were already set.","ref":"Cachex.html#put/4"},{"type":"function","title":"Options - Cachex.put/4","doc":"* `:expire`\n\n    An expiration value to set for the provided key (time-to-live), overriding\n    any default expirations set on a cache. This value should be in milliseconds.","ref":"Cachex.html#put/4-options"},{"type":"function","title":"Examples - Cachex.put/4","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    { :ok, true }\n\n    iex> Cachex.put(:my_cache, \"key\", \"value\", expire: :timer.seconds(5))\n    iex> Cachex.ttl(:my_cache, \"key\")\n    { :ok, 5000 }","ref":"Cachex.html#put/4-examples"},{"type":"function","title":"Cachex.put_many/3","doc":"Places a batch of entries in a cache.\n\nThis operates in the same way as `put/4`, except that multiple keys can be\ninserted in a single atomic batch. This is a performance gain over writing\nkeys using multiple calls to `put/4`, however it's a performance penalty\nwhen writing a single key pair due to some batching overhead.","ref":"Cachex.html#put_many/3"},{"type":"function","title":"Options - Cachex.put_many/3","doc":"* `:expire`\n\n    An expiration value to set for the provided keys (time-to-live), overriding\n    any default expirations set on a cache. This value should be in milliseconds.","ref":"Cachex.html#put_many/3-options"},{"type":"function","title":"Examples - Cachex.put_many/3","doc":"iex> Cachex.put_many(:my_cache, [ { \"key\", \"value\" } ])\n    { :ok, true }\n\n    iex> Cachex.put_many(:my_cache, [ { \"key\", \"value\" } ], expire: :timer.seconds(5))\n    iex> Cachex.ttl(:my_cache, \"key\")\n    { :ok, 5000 }","ref":"Cachex.html#put_many/3-examples"},{"type":"function","title":"Cachex.refresh/3","doc":"Refreshes an expiration for an entry in a cache.\n\nRefreshing an expiration will reset the existing expiration with an offset\nfrom the current time - i.e. if you set an expiration of 5 minutes and wait\n3 minutes before refreshing, the entry will expire 8 minutes after the initial\ninsertion.","ref":"Cachex.html#refresh/3"},{"type":"function","title":"Examples - Cachex.refresh/3","doc":"iex> Cachex.put(:my_cache, \"my_key\", \"my_value\", expire: :timer.seconds(5))\n    iex> Process.sleep(4)\n    iex> Cachex.ttl(:my_cache, \"my_key\")\n    { :ok, 1000 }\n\n    iex> Cachex.refresh(:my_cache, \"my_key\")\n    iex> Cachex.ttl(:my_cache, \"my_key\")\n    { :ok, 5000 }\n\n    iex> Cachex.refresh(:my_cache, \"missing_key\")\n    { :ok, false }","ref":"Cachex.html#refresh/3-examples"},{"type":"function","title":"Cachex.reset/2","doc":"Resets a cache by clearing the keyspace and restarting any hooks.","ref":"Cachex.html#reset/2"},{"type":"function","title":"Options - Cachex.reset/2","doc":"* `:hooks`\n\n    A whitelist of hooks to reset on the cache instance (call the\n    initialization phase of a hook again). This will default to\n    resetting all hooks associated with a cache, which is usually\n    the desired behaviour.\n\n  * `:only`\n\n    A whitelist of components to reset, which can currently contain\n    either the `:cache` or `:hooks` tag to determine what to reset.\n    This will default to `[ :cache, :hooks ]`.","ref":"Cachex.html#reset/2-options"},{"type":"function","title":"Examples - Cachex.reset/2","doc":"iex> Cachex.put(:my_cache, \"my_key\", \"my_value\")\n    iex> Cachex.reset(:my_cache)\n    iex> Cachex.size(:my_cache)\n    { :ok, 0 }\n\n    iex> Cachex.reset(:my_cache, [ only: :hooks ])\n    { :ok, true }\n\n    iex> Cachex.reset(:my_cache, [ only: :hooks, hooks: [ MyHook ] ])\n    { :ok, true }\n\n    iex> Cachex.reset(:my_cache, [ only: :cache ])\n    { :ok, true }","ref":"Cachex.html#reset/2-examples"},{"type":"function","title":"Cachex.restore/3","doc":"Deserializes a cache from a location on a filesystem.\n\nThis operation will read the current state of a cache from a provided\nlocation on a filesystem. This function will only understand files\nwhich have previously been created using `Cachex.save/3`.\n\nIt is the responsibility of the user to ensure that the location is\nable to be read from, not the responsibility of Cachex.","ref":"Cachex.html#restore/3"},{"type":"function","title":"Options - Cachex.restore/3","doc":"* `:trust`\n\n    Allow for loading from trusted or untrusted sources; trusted\n    sources can load atoms into the table, whereas untrusted sources\n    cannot. Defaults to `true`.","ref":"Cachex.html#restore/3-options"},{"type":"function","title":"Examples - Cachex.restore/3","doc":"iex> Cachex.put(:my_cache, \"my_key\", 10)\n    iex> Cachex.save(:my_cache, \"/tmp/my_backup\")\n    { :ok, true }\n\n    iex> Cachex.size(:my_cache)\n    { :ok, 1 }\n\n    iex> Cachex.clear(:my_cache)\n    iex> Cachex.size(:my_cache)\n    { :ok, 0 }\n\n    iex> Cachex.restore(:my_cache, \"/tmp/my_backup\")\n    { :ok, 1 }\n\n    iex> Cachex.size(:my_cache)\n    { :ok, 1 }","ref":"Cachex.html#restore/3-examples"},{"type":"function","title":"Cachex.save/3","doc":"Serializes a cache to a location on a filesystem.\n\nThis operation will write the current state of a cache to a provided\nlocation on a filesystem. The written state can be used alongside the\n`Cachex.restore/3` command to import back in the future.\n\nIt is the responsibility of the user to ensure that the location is\nable to be written to, not the responsibility of Cachex.","ref":"Cachex.html#save/3"},{"type":"function","title":"Options - Cachex.save/3","doc":"* `:buffer`\n\n    Allows customization of the internal batching when paginating the cursor\n    coming back from ETS. It's unlikely this will ever need changing.","ref":"Cachex.html#save/3-options"},{"type":"function","title":"Examples - Cachex.save/3","doc":"iex> Cachex.save(:my_cache, \"/tmp/my_default_backup\")\n    { :ok, true }","ref":"Cachex.html#save/3-examples"},{"type":"function","title":"Cachex.size/2","doc":"Retrieves the total size of a cache.\n\nBy default this does not take expiration time of entries inside the\ncache into account, making it an `O(1)` call. This behaviour can be\nmodified by passing `expired: false` to account for expirations, but\nplease note that this is a much more involved calculation.","ref":"Cachex.html#size/2"},{"type":"function","title":"Options - Cachex.size/2","doc":"* `:expired`\n\n    Whether or not to include expired records in the returned total. This\n    is a boolean value which defaults to `true`.","ref":"Cachex.html#size/2-options"},{"type":"function","title":"Examples - Cachex.size/2","doc":"iex> Cachex.put(:my_cache, \"key1\", \"value1\")\n    iex> Cachex.put(:my_cache, \"key2\", \"value2\")\n    iex> Cachex.put(:my_cache, \"key3\", \"value3\", expire: 1)\n    iex> Cachex.size(:my_cache)\n    { :ok, 3 }\n\n    iex> Cachex.size(:my_cache, expired: false)\n    { :ok, 2 }","ref":"Cachex.html#size/2-examples"},{"type":"function","title":"Cachex.start/2","doc":"Creates a new Cachex cache service tree.\n\nThis will not link the cache to the current process, so if your process dies\nthe cache will continue to live. If you don't want this behaviour, please use\nthe provided `Cachex.start_link/2`.\n\nThis function is otherwise identical to `start_link/2` so please see that\ndocumentation for further information and configuration.","ref":"Cachex.html#start/2"},{"type":"function","title":"Cachex.start_link/2","doc":"Creates a new Cachex cache service tree, linked to the current process.\n\nThis will link the cache to the current process, so if your process dies the\ncache will also die. If you don't want this behaviour, please use `start/2`.\n\nThe first argument should be a unique atom, used as the name of the cache\nservice for future calls through to Cachex. For all options requiring a record\nargument, please import `Cachex.Spec` in advance.","ref":"Cachex.html#start_link/2"},{"type":"function","title":"Options - Cachex.start_link/2","doc":"* `:commands`\n\n    This option allows you to attach a set of custom commands to a cache in\n    order to provide shorthand execution. A cache command must be constructed\n    using the `:command` record provided by `Cachex.Spec`.\n\n    A cache command will adhere to these basic rules:\n\n    - If you define a `:read` command, the return value of your command will\n      be passed through as the result of your call to `invoke/4`.\n    - If you define a `:write` command, your command must return a two-element\n      Tuple. The first element represents the value being returned from your\n      `invoke/4` call, and the second represents the value to write back into\n      the cache (as an update). If your command does not fit this, errors will\n      happen (intentionally).\n\n    Commands are set on a per-cache basis, but can be reused across caches. They're\n    set only on cache startup and cannot be modified after the cache tree is created.\n\n        iex> import Cachex.Spec\n        ...>\n        ...> Cachex.start_link(:my_cache, [\n        ...>   commands: [\n        ...>     last: command(type:  :read, execute:   &List.last/1),\n        ...>     trim: command(type: :write, execute: &String.trim/1)\n        ...>   ]\n        ...> ])\n        { :ok, _pid }\n\n    Either a `Keyword` or a `Map` can be provided against the `:commands` option as\n    we only use `Enum` to verify them before attaching them internally. Please see\n    the `Cachex.Spec.command/1` documentation for further customization options.\n\n  * `:compressed`\n\n    This option will specify whether this cache should have enable ETS compression,\n    which is likely to reduce memory overhead. Please note that there is a potential\n    for this option to slow your cache due to compression overhead, so benchmark as\n    appropriate when using this option. This option defaults to `false`.\n\n        iex> Cachex.start_link(:my_cache, [ compressed: true ])\n        { :ok, _pid }\n\n  * `:expiration`\n\n    The expiration option provides the ability to customize record expiration at\n    a global cache level. The value provided here must be a valid `:expiration`\n    record provided by `Cachex.Spec`.\n\n        iex> import Cachex.Spec\n        ...>\n        ...> Cachex.start_link(:my_cache, [\n        ...>   expiration: expiration(\n        ...>     # how often cleanup should occur\n        ...>     interval: :timer.seconds(30),\n        ...>\n        ...>     # default record expiration\n        ...>     default: :timer.seconds(60),\n        ...>\n        ...>     # whether to enable lazy checking\n        ...>     lazy: true\n        ...>   )\n        ...> ])\n        { :ok, _pid }\n\n    Please see the `Cachex.Spec.expiration/1` documentation for further customization\n    options.\n\n  * `:hooks`\n\n    The `:hooks` option allow the user to attach a list of notification hooks to\n    enable listening on cache actions (either before or after they happen). These\n    hooks should be valid `:hook` records provided by `Cachex.Spec`. Example hook\n    implementations can be found in `Cachex.Stats` and `Cachex.Policy.LRW`.\n\n        iex> import Cachex.Spec\n        ...>\n        ...> Cachex.start_link(:my_cache, [\n        ...>   hooks: [\n        ...>     hook(module: MyHook, name: :my_hook, args: { })\n        ...>   ]\n        ...> ])\n        { :ok, _pid }\n\n    Please see the `Cachex.Spec.hook/1` documentation for further customization options.\n\n  * `:ordered`\n\n    This option will specify whether this cache should enable ETS ordering, which can\n    improve performance if ordered traversal of a cache is required. Setting `:ordered`\n    to `true` will result in both insert and lookup times being proportional to the\n    logarithm of the number of objects in the table. This option defaults to `false`.\n\n        iex> Cachex.start_link(:my_cache, [ ordered: true ])\n        { :ok, _pid }\n\n  * `:router`\n\n    This option determines which module is used for cache routing inside distributed\n    caches. You can provide either a full `record` structure or simply a module name.\n\n        iex> import Cachex.Spec\n        ...>\n        ...> Cachex.start_link(:my_cache, [\n        ...>   router: router(\n        ...>     module: Cachex.Router.Jump,\n        ...>     options: []\n        ...>   )\n        ...> ])\n        { :ok, _pid }\n\n    Please see the `Cachex.Spec.router/1` documentation for further customization options.\n\n  * `:transactions`\n\n    This option will specify whether this cache should have transactions and row\n    locking enabled from cache startup. Please note that even if this is false,\n    it will be enabled the moment a transaction is executed. It's recommended to\n    leave this as default as it will handle most use cases in the most performant\n    way possible.\n\n        iex> Cachex.start_link(:my_cache, [ transactions: true ])\n        { :ok, _pid }\n\n  * `:warmers`\n\n    The `:warmers` option allows the user to attach a list of warming modules to\n    a cache. These cache warmers must implement the `Cachex.Warmer` behaviour\n    and are defined as `warmer` records.\n\n    The only required value is the `:module` definition, although you can also\n    choose to provide a name and state to attach to the warmer process. The flag\n    `:required` is used to control whether the warmer must finish execution before\n    the cache supervision tree can be considered fully started.\n\n        iex> import Cachex.Spec\n        ...>\n        ...> Cachex.start_link(:my_cache, [\n        ...>   warmers: [\n        ...>     warmer(\n        ...>       required: true,\n        ...>       module: MyProject.DatabaseWarmer,\n        ...>       state: connection,\n        ...>       name: MyProject.DatabaseWarmer\n        ...>     )\n        ...>   ]\n        ...> ])\n        { :ok, _pid }\n\n    Please see the `Cachex.Spec.warmer/1` documentation for further customization options.","ref":"Cachex.html#start_link/2-options"},{"type":"function","title":"Cachex.stats/2","doc":"Retrieves statistics about a cache.\n\nThis will only provide statistics if the `:stats` option was\nprovided on cache startup in `start_link/2`.","ref":"Cachex.html#stats/2"},{"type":"function","title":"Examples - Cachex.stats/2","doc":"iex> Cachex.stats(:my_cache)\n    {:ok, %{meta: %{creation_date: 1518984857331}}}\n\n    iex> Cachex.stats(:cache_with_no_stats)\n    { :error, :stats_disabled }","ref":"Cachex.html#stats/2-examples"},{"type":"function","title":"Cachex.stream/3","doc":"Creates a `Stream` of entries in a cache.\n\nThis will stream all entries matching the match specification provided\nas the second argument. If none is provided, it will default to all entries\nwhich are yet to expire (in no particular order).\n\nConsider using `Cachex.Query` to generate match specifications used when\nquerying the contents of a cache table.","ref":"Cachex.html#stream/3"},{"type":"function","title":"Options - Cachex.stream/3","doc":"* `:buffer`\n\n    Allows customization of the internal batching when paginating the cursor\n    coming back from ETS. It's unlikely this will ever need changing.","ref":"Cachex.html#stream/3-options"},{"type":"function","title":"Examples - Cachex.stream/3","doc":"iex> Cachex.put(:my_cache, \"a\", 1)\n    iex> Cachex.put(:my_cache, \"b\", 2)\n    iex> Cachex.put(:my_cache, \"c\", 3)\n    {:ok, true}\n\n    iex> :my_cache |> Cachex.stream! |> Enum.to_list\n    [{:entry, \"b\", 1519015801794, nil, 2},\n      {:entry, \"c\", 1519015805679, nil, 3},\n      {:entry, \"a\", 1519015794445, nil, 1}]\n\n    iex> query = Cachex.Query.build(output: :key)\n    iex> :my_cache |> Cachex.stream!(query) |> Enum.to_list\n    [\"b\", \"c\", \"a\"]\n\n    iex> query = Cachex.Query.build(output: :value)\n    iex> :my_cache |> Cachex.stream!(query) |> Enum.to_list\n    [2, 3, 1]\n\n    iex> query = Cachex.Query.build(output: {:key, :value})\n    iex> :my_cache |> Cachex.stream!(query) |> Enum.to_list\n    [{\"b\", 2}, {\"c\", 3}, {\"a\", 1}]","ref":"Cachex.html#stream/3-examples"},{"type":"function","title":"Cachex.take/3","doc":"Takes an entry from a cache.\n\nThis is conceptually equivalent to running `get/3` followed\nby an atomic `del/3` call.","ref":"Cachex.html#take/3"},{"type":"function","title":"Examples - Cachex.take/3","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.take(:my_cache, \"key\")\n    { :ok, \"value\" }\n\n    iex> Cachex.get(:my_cache, \"key\")\n    { :ok, nil }\n\n    iex> Cachex.take(:my_cache, \"missing_key\")\n    { :ok, nil }","ref":"Cachex.html#take/3-examples"},{"type":"function","title":"Cachex.touch/3","doc":"Updates the last write time on a cache entry.\n\nThis is very similar to `refresh/3` except that the expiration\ntime is maintained inside the record (using a calculated offset).","ref":"Cachex.html#touch/3"},{"type":"function","title":"Cachex.transaction/4","doc":"Executes multiple functions in the context of a transaction.\n\nThis will operate in the same way as `execute/3`, except that writes\nto the specified keys will be blocked on the execution of this transaction.\n\nThe keys parameter should be a list of keys you wish to lock whilst\nyour transaction is executed. Any keys not in this list can still be\nwritten even during your transaction.","ref":"Cachex.html#transaction/4"},{"type":"function","title":"Examples - Cachex.transaction/4","doc":"iex> Cachex.put(:my_cache, \"key1\", \"value1\")\n    iex> Cachex.put(:my_cache, \"key2\", \"value2\")\n    iex> Cachex.transaction(:my_cache, [ \"key1\", \"key2\" ], fn(worker) ->\n    ...>   val1 = Cachex.get(worker, \"key1\")\n    ...>   val2 = Cachex.get(worker, \"key2\")\n    ...>   [val1, val2]\n    ...> end)\n    { :ok, [ \"value1\", \"value2\" ] }","ref":"Cachex.html#transaction/4-examples"},{"type":"function","title":"Cachex.ttl/3","doc":"Retrieves the expiration for an entry in a cache.\n\nThis is a millisecond value (if set) representing the time a\ncache entry has left to live in a cache. It can return `nil`\nif the entry does not have a set expiration.","ref":"Cachex.html#ttl/3"},{"type":"function","title":"Examples - Cachex.ttl/3","doc":"iex> Cachex.ttl(:my_cache, \"my_key\")\n    { :ok, 13985 }\n\n    iex> Cachex.ttl(:my_cache, \"my_key_with_no_ttl\")\n    { :ok, nil }\n\n    iex> Cachex.ttl(:my_cache, \"missing_key\")\n    { :ok, nil }","ref":"Cachex.html#ttl/3-examples"},{"type":"function","title":"Cachex.update/4","doc":"Updates an entry in a cache.\n\nUnlike `get_and_update/4`, this does a blind overwrite of a value.\n\nThis operation can be seen as an internal mutation, meaning that any previously\nset expiration time is kept as-is.","ref":"Cachex.html#update/4"},{"type":"function","title":"Examples - Cachex.update/4","doc":"iex> Cachex.put(:my_cache, \"key\", \"value\")\n    iex> Cachex.get(:my_cache, \"key\")\n    { :ok, \"value\" }\n\n    iex> Cachex.update(:my_cache, \"key\", \"new_value\")\n    iex> Cachex.get(:my_cache, \"key\")\n    { :ok, \"new_value\" }\n\n    iex> Cachex.update(:my_cache, \"missing_key\", \"new_value\")\n    { :ok, false }","ref":"Cachex.html#update/4-examples"},{"type":"function","title":"Cachex.warm/2","doc":"Triggers a manual warming in a cache.\n\nThis allows for manual warming of a cache in situations where\nyou already know the backing state has been updated. The return\nvalue of this function will contain the list of modules which\nwere warmed as a result of this call.","ref":"Cachex.html#warm/2"},{"type":"function","title":"Options - Cachex.warm/2","doc":"* `:only`\n\n    An optional list of modules to warm, acting as a whitelist. The default\n    behaviour of this function is to trigger warming in all modules. You may\n    provide either the module name, or the registered warmer name.\n\n  * `:wait`\n\n    Whether to wait for warmer completion or not, as a boolean. By default\n    warmers are triggered to run in the background, but passing `true` here\n    will block the return of this call until all warmers have completed.","ref":"Cachex.html#warm/2-options"},{"type":"function","title":"Examples - Cachex.warm/2","doc":"iex> Cachex.warm(:my_cache)\n    { :ok, [MyWarmer] }\n\n    iex> Cachex.warm(:my_cache, only: [MyWarmer])\n    { :ok, [MyWarmer] }\n\n    iex> Cachex.warm(:my_cache, only: [])\n    { :ok, [] }\n\n    iex> Cachex.warm(:my_cache, wait: true)\n    { :ok, [MyWarmer]}","ref":"Cachex.html#warm/2-examples"},{"type":"type","title":"Cachex.status/0","doc":"","ref":"Cachex.html#t:status/0"},{"type":"type","title":"Cachex.t/0","doc":"","ref":"Cachex.html#t:t/0"},{"type":"behaviour","title":"Cachex.Hook","doc":"Module controlling hook behaviour definitions.\n\nThis module defines the hook implementations for Cachex, allowing the user to\nadd hooks into the command execution. This means that users can build plugin\nstyle listeners in order to do things like logging. Hooks can be registered\nto execute either before or after the Cachex command, and can be blocking as\nneeded.","ref":"Cachex.Hook.html"},{"type":"callback","title":"Cachex.Hook.actions/0","doc":"Returns the actions this hook is expected to listen on.\n\nThis will default to an empty list, to force the developer to opt into the\nactions they receive notifications for. If all actions should be received,\nyou can use the `:all` atom to receive everything.","ref":"Cachex.Hook.html#c:actions/0"},{"type":"callback","title":"Cachex.Hook.async?/0","doc":"Returns whether this hook is asynchronous or not.","ref":"Cachex.Hook.html#c:async?/0"},{"type":"function","title":"Cachex.Hook.concat/1","doc":"Concatenates all hooks in a cache.","ref":"Cachex.Hook.html#concat/1"},{"type":"callback","title":"Cachex.Hook.handle_notify/3","doc":"Handles a cache notification.\n\nThe first argument is the action being taken along with arguments, with the\nsecond argument being the results of the action (this can be nil for hooks)\nwhich fire before the action is executed.","ref":"Cachex.Hook.html#c:handle_notify/3"},{"type":"function","title":"Cachex.Hook.locate/3","doc":"Locates a hook module for a cache.","ref":"Cachex.Hook.html#locate/3"},{"type":"callback","title":"Cachex.Hook.timeout/0","doc":"Returns the timeout for all calls to this hook.\n\nThis will be applied to hooks regardless of whether they're synchronous or\nnot; a behaviour change which shipped in v3.0 initially.","ref":"Cachex.Hook.html#c:timeout/0"},{"type":"callback","title":"Cachex.Hook.type/0","doc":"Returns the type of this hook.\n\nThis should return `:post` to fire after a cache action has occurred, and\nreturn `:pre` if it should fire before the action occurs. The `:quiet` type\nis for hooks which don't listen to any broadcasts.","ref":"Cachex.Hook.html#c:type/0"},{"type":"function","title":"Cachex.Hook.types/0","doc":"Retrieve all known types of hook.","ref":"Cachex.Hook.html#types/0"},{"type":"module","title":"Cachex.Limit.Accessed","doc":"Access based touch tracking for LRW pruning.\n\nThis module can be used to adapt `Cachex.prune/3` for LRU purposes instead of\nthe typical LRW. This hook will update the modification time of a cache entry\nupon access by a read operation. This is a very basic way to provide LRU policies,\nbut it should suffice for most cases.\n\nAt the time of writing modification times are *not* updated when executing\ncommands on multiple keys, such as `Cachex.keys/2` and `Cachex.stream/3`, for\nperformance reasons. Again, this may change in future if necessary.","ref":"Cachex.Limit.Accessed.html"},{"type":"function","title":"Cachex.Limit.Accessed.actions/0","doc":"Returns the actions this hook should listen on.","ref":"Cachex.Limit.Accessed.html#actions/0"},{"type":"function","title":"Cachex.Limit.Accessed.provisions/0","doc":"Returns the provisions this hook requires.","ref":"Cachex.Limit.Accessed.html#provisions/0"},{"type":"module","title":"Cachex.Limit.Evented","doc":"Evented least recently written eviction policy for Cachex.\n\nThis module implements an evented LRW eviction policy for Cachex, using a hook\nto listen for new key additions to a cache and enforcing bounds in a reactive\nway. This policy enforces cache bounds and limits far more accurately than other\nscheduled implementations, but comes at a higher memory cost (due to the message\npassing between hooks).","ref":"Cachex.Limit.Evented.html"},{"type":"module","title":"Initialization - Cachex.Limit.Evented","doc":"hook(module: Cachex.Limit.Evented, args: {\n      500,  # setting cache max size\n      []    # options for `Cachex.prune/3`\n    })","ref":"Cachex.Limit.Evented.html#module-initialization"},{"type":"function","title":"Cachex.Limit.Evented.actions/0","doc":"Returns the actions this policy should listen on.","ref":"Cachex.Limit.Evented.html#actions/0"},{"type":"function","title":"Cachex.Limit.Evented.provisions/0","doc":"Returns the provisions this policy requires.","ref":"Cachex.Limit.Evented.html#provisions/0"},{"type":"module","title":"Cachex.Limit.Scheduled","doc":"Scheduled least recently written eviction policy for Cachex.\n\nThis module implements a scheduled LRW eviction policy for Cachex, using a basic\ntimer to trigger bound enforcement in a repeatable way. This has the same bound\naccuracy as `Cachex.Policy.LRW.Evented`, but has potential for some delay. The\nmain advantage of this implementation is a far lower memory cost due to not\nusing hook messages.","ref":"Cachex.Limit.Scheduled.html"},{"type":"module","title":"Initialization - Cachex.Limit.Scheduled","doc":"hook(module: Cachex.Limit.Scheduled, args: {\n      500,  # setting cache max size\n      [],   # options for `Cachex.prune/3`\n      []    # options for `Cachex.Limit.Scheduled`\n    })","ref":"Cachex.Limit.Scheduled.html#module-initialization"},{"type":"module","title":"Options - Cachex.Limit.Scheduled","doc":"* `:frequency`\n\n    The polling frequency for this hook to use when scheduling cache pruning.\n    This should be an non-negative number of milliseconds. Defaults to `1000`,\n    which is once per second.","ref":"Cachex.Limit.Scheduled.html#module-options"},{"type":"function","title":"Cachex.Limit.Scheduled.provisions/0","doc":"Returns the provisions this policy requires.","ref":"Cachex.Limit.Scheduled.html#provisions/0"},{"type":"behaviour","title":"Cachex.Provision","doc":"Module controlling provisioning behaviour definitions.\n\nThis module defines the provisioning implementation for Cachex, allowing\ncomponents such as hooks and warmers to tap into state changes in the cache\ntable. By implementing `handle_provision/2` these components can be provided\nwith new versions of state as they're created.","ref":"Cachex.Provision.html"},{"type":"callback","title":"Cachex.Provision.handle_provision/2","doc":"Handles a provisioning call.\n\nThe provided argument will be a Tuple dictating the type of value being\nprovisioned along with the value itself. This can be used to listen on\nstates required for hook executions (such as cache records).","ref":"Cachex.Provision.html#c:handle_provision/2"},{"type":"callback","title":"Cachex.Provision.provisions/0","doc":"Returns an enumerable of provisions this implementation requires.\n\nThe current provisions available are:\n\n  * `cache` - a cache instance used to make cache calls with lower overhead.\n\nThis should always return an enumerable of atoms; in the case of no required\nprovisions an empty enumerable should be returned.","ref":"Cachex.Provision.html#c:provisions/0"},{"type":"module","title":"Cachex.Query","doc":"Utility module based around creation of cache queries.\n\nQueries are essentially just some minor convenience wrappers around the\ninternal match specification used by ETS. This module is exposed to make\nit easier to query a cache (via `Cachex.stream/3`) without having to take\ncare of filtering for expirations by hand.\n\nNote that there is almost no validation in here, so test thoroughly and\nstore compile-time versions of your queries when possible (as performance\nis not taken into account inside this module; it can be slow to generate).","ref":"Cachex.Query.html"},{"type":"function","title":"Cachex.Query.build/1","doc":"Create a query specification for a cache.\n\nThis is a convenience binding to ETS select specifications, so please see\nthe appropriate documentation for any additional information on how to (e.g.)\nformat filter clauses and outputs.","ref":"Cachex.Query.html#build/1"},{"type":"function","title":"Options - Cachex.Query.build/1","doc":"* `:output`\n\n    The query output format, which defaults to `:entry` (retrieving the entire\n    cache entry). You can provide any of the entry record fields, with bindings\n    such as tuples (`{:key, :value}`) or lists `[:key, :value]` or as a single\n    value (`:key`).\n\n  * `:where`\n\n    An ETS filter condition used to locate records in the table. This defaults to\n    retrieving all records, and is automatically joined with the value of the\n    `:expired` flag for convenience.","ref":"Cachex.Query.html#build/1-options"},{"type":"function","title":"Cachex.Query.expired/1","doc":"Create a filter against expired records in a cache.\n\nThis function accepts a subfilter to join to create more complex filters.","ref":"Cachex.Query.html#expired/1"},{"type":"function","title":"Cachex.Query.unexpired/1","doc":"Create a filter against unexpired records in a cache.\n\nThis function accepts a subfilter to join to create more complex filters.","ref":"Cachex.Query.html#unexpired/1"},{"type":"behaviour","title":"Cachex.Router","doc":"Module controlling routing behaviour definitions.\n\nThis module defines the router implementations for Cachex, allowing the user\nto route commands between nodes in a cache cluster. This means that users\ncan provide their own routing and rebalancing logic without having to depend\non it being included in Cachex.","ref":"Cachex.Router.html"},{"type":"callback","title":"Cachex.Router.children/2","doc":"Create a child specification to back a routing state.","ref":"Cachex.Router.html#c:children/2"},{"type":"function","title":"Cachex.Router.connected/0","doc":"Retrieve all currently connected nodes (including this one).","ref":"Cachex.Router.html#connected/0"},{"type":"callback","title":"Cachex.Router.init/2","doc":"Initialize a routing state for a cache.\n\nPlease see all child implementations for supported options.","ref":"Cachex.Router.html#c:init/2"},{"type":"function","title":"Cachex.Router.nodes/1","doc":"Retrieve all routable nodes for a cache.","ref":"Cachex.Router.html#nodes/1"},{"type":"callback","title":"Cachex.Router.nodes/1","doc":"Retrieve the list of nodes from a routing state.","ref":"Cachex.Router.html#c:nodes/1"},{"type":"macro","title":"Cachex.Router.route/2","doc":"Dispatches a call to an appropriate execution environment.\n\nThis acts as a macro just to avoid the overhead of slicing up module\nnames at runtime, when they can be guaranteed at compile time much\nmore easily.","ref":"Cachex.Router.html#route/2"},{"type":"callback","title":"Cachex.Router.route/2","doc":"Route a key to a node in a routing state.","ref":"Cachex.Router.html#c:route/2"},{"type":"function","title":"Cachex.Router.route/3","doc":"Executes a previously dispatched action..","ref":"Cachex.Router.html#route/3"},{"type":"module","title":"Cachex.Router.Jump","doc":"Routing implementation based on Jump Consistent Hash.\n\nThis implementation backed Cachex's distribution in the v3.x lineage, and is\nsuitable for clusters of a static size. Each key is hashed and then slotted\nagainst a node in the cluster. Please note that the hash algorithm should\nnot be relied upon and is not considered part of the public API.\n\nThe initialization of this router accepts a `:nodes` option which enables\nthe user to define the nodes to route amongst. If this is not provided the\nrouter will default to detecting a cluster via `Node.self/0` and `Node.list/1`.\n\nFor more information on the algorithm backing this router, please see the\nappropriate [publication](https://arxiv.org/pdf/1406.2294).","ref":"Cachex.Router.Jump.html"},{"type":"function","title":"Cachex.Router.Jump.init/2","doc":"Initialize a jump hash routing state for a cache.","ref":"Cachex.Router.Jump.html#init/2"},{"type":"function","title":"Options - Cachex.Router.Jump.init/2","doc":"* `:nodes`\n\n    The `:nodes` option allows a user to provide a list of nodes to treat\n    as a cluster. If this is not provided, the cluster will be inferred\n    by using `Node.self/0` and `Node.list/1`.","ref":"Cachex.Router.Jump.html#init/2-options"},{"type":"function","title":"Cachex.Router.Jump.nodes/1","doc":"Retrieve the list of nodes from a jump hash routing state.","ref":"Cachex.Router.Jump.html#nodes/1"},{"type":"function","title":"Cachex.Router.Jump.route/2","doc":"Route a key to a node in a jump hash routing state.","ref":"Cachex.Router.Jump.html#route/2"},{"type":"module","title":"Cachex.Router.Local","doc":"Routing implementation for the local node.\n\nThis module acts as the base implementation for routing when *not* being\nused in a distributed cache. All actions are routed to the current node.","ref":"Cachex.Router.Local.html"},{"type":"function","title":"Cachex.Router.Local.nodes/1","doc":"Retrieve the list of nodes from a local routing state.","ref":"Cachex.Router.Local.html#nodes/1"},{"type":"function","title":"Cachex.Router.Local.route/2","doc":"Route a key to a node in a local routing state.","ref":"Cachex.Router.Local.html#route/2"},{"type":"module","title":"Cachex.Router.Mod","doc":"Routing implementation based on basic hashing.\n\nThis router provides the simplest (and quickest!) implementation for\nclusters of a static size. Provided keys are hashed and routed to a node\nvia the modulo operation. Please note that the hash algorithm should\nnot be relied upon and is not considered part of the public API.\n\nThe initialization of this router accepts a `:nodes` option which enables\nthe user to define the nodes to route amongst. If this is not provided the\nrouter will default to detecting a cluster via `Node.self/0` and `Node.list/1`.","ref":"Cachex.Router.Mod.html"},{"type":"function","title":"Cachex.Router.Mod.init/2","doc":"Initialize a modulo routing state for a cache.","ref":"Cachex.Router.Mod.html#init/2"},{"type":"function","title":"Options - Cachex.Router.Mod.init/2","doc":"* `:nodes`\n\n    The `:nodes` option allows a user to provide a list of nodes to treat\n    as a cluster. If this is not provided, the cluster will be inferred\n    by using `Node.self/0` and `Node.list/1`.","ref":"Cachex.Router.Mod.html#init/2-options"},{"type":"function","title":"Cachex.Router.Mod.nodes/1","doc":"Retrieve the list of nodes from a modulo routing state.","ref":"Cachex.Router.Mod.html#nodes/1"},{"type":"function","title":"Cachex.Router.Mod.route/2","doc":"Route a key to a node in a modulo routing state.","ref":"Cachex.Router.Mod.html#route/2"},{"type":"module","title":"Cachex.Router.Ring","doc":"Routing implementation using a consistent hash ring.\n\nThis router provides the most resilient routing for a distributed cache,\ndue to being much more resilient to addition and removal of nodes in\nthe cluster. Most distributed caches will end up using this router if\nthey have the requirement to handle such cases.\n\nThe core implementation of this router is provided by the Discord\n[library](https://github.com/discord/ex_hash_ring), so please see their\nrepository for further details.","ref":"Cachex.Router.Ring.html"},{"type":"function","title":"Cachex.Router.Ring.children/2","doc":"Create a child specification to back a ring routing state.","ref":"Cachex.Router.Ring.html#children/2"},{"type":"function","title":"Cachex.Router.Ring.included?/3","doc":"Returns whether a node is included given the provided patterns.","ref":"Cachex.Router.Ring.html#included?/3"},{"type":"function","title":"Cachex.Router.Ring.init/2","doc":"Initialize a ring routing state for a cache.","ref":"Cachex.Router.Ring.html#init/2"},{"type":"function","title":"Options - Cachex.Router.Ring.init/2","doc":"* `:nodes`\n\n    The `:nodes` option allows a user to provide a list of nodes to treat\n    as a cluster. If this is not provided, the cluster will be inferred\n    by using `Node.self/0` and `Node.list/1`.\n\n  * `:monitor`\n\n    This option specifies whether to monitor Erlang `:nodeup` and `:nodedown`\n    events and scale this ring to add/remove nodes dynamically. This defaults\n    to `false`, but if you're using this router it's likely you want this enabled.\n\n  * `:monitor_excludes`\n\n    This option allows the developer to provide a list of patterns to exclude\n    nodes from joining automatically when monitoring is enabled. Patterns can be\n    provided as either binaries or Regex.\n\n  * `:monitor_includes`\n\n    This option allows the developer to provide a list of patterns to validate\n    nodes before allowing them to join the ring. Patterns can be provided as\n    either binaries or Regex.\n\n  * `:monitor_type`\n\n    The type of nodes to listen for and dynamically add to the internal ring.\n    This defaults to `:all`, but can be any value accepted by the OTP function\n    `:net_kernel.monitor_nodes/2`.","ref":"Cachex.Router.Ring.html#init/2-options"},{"type":"function","title":"Cachex.Router.Ring.nodes/1","doc":"Retrieve the list of nodes from a ring routing state.","ref":"Cachex.Router.Ring.html#nodes/1"},{"type":"function","title":"Cachex.Router.Ring.route/2","doc":"Route a key to a node in a ring routing state.","ref":"Cachex.Router.Ring.html#route/2"},{"type":"module","title":"Cachex.Spec","doc":"Specification definitions based around records and utilities.\n\nThis serves as the \"parent\" header file for Cachex, where all records\nand macros are located. It's designed as a single inclusion file which\nprovides everything you might need to implement features in Cachex, and\nindeed when interacting with Cachex.\n\nMost macros in here should be treated as reserved for internal use only,\nbut those based around records can be freely used by consumers of Cachex.","ref":"Cachex.Spec.html"},{"type":"macro","title":"Cachex.Spec.cache/1","doc":"Creates a cache record from the provided values.\n\nA cache record is used to represent the internal state of a cache, and is used\nwhen executing calls. Most values in here will be other records defined in the\nmain specification, and as such please see their documentation for further info.","ref":"Cachex.Spec.html#cache/1"},{"type":"macro","title":"Cachex.Spec.cache/2","doc":"Updates a cache record from the provided values.","ref":"Cachex.Spec.html#cache/2"},{"type":"macro","title":"Cachex.Spec.command/1","doc":"Creates a command record from the provided values.\n\nA command is a custom action which can be executed against a cache instance. They\nconsist of a type (`:read`/`:write`) and an execution function. The type determines\nwhat form the execution should take.\n\nIn the case of a `:read` type, an execution function is a simple (any -> any) form,\nwhich will return the returned value directly to the caller. In the case of a `:write`\ntype, an execution should be (any -> { any, any }) where the value in the left side\nof the returned Tuple will be returned to the caller, and the right side will be set\ninside the backing cache table.","ref":"Cachex.Spec.html#command/1"},{"type":"macro","title":"Cachex.Spec.command/2","doc":"Updates a command record from the provided values.","ref":"Cachex.Spec.html#command/2"},{"type":"macro","title":"Cachex.Spec.entry/1","doc":"Creates an entry record from the provided values.\n\nAn entry record represents a single entry in a cache table.\n\nEach entry has a key/value, along with a touch time and ttl. These records should never\nbe used outside of the Cachex codebase other than when debugging, as they can change\nat any time and should be regarded as internal only.","ref":"Cachex.Spec.html#entry/1"},{"type":"macro","title":"Cachex.Spec.entry/2","doc":"Updates an entry record from the provided values.","ref":"Cachex.Spec.html#entry/2"},{"type":"macro","title":"Cachex.Spec.expiration/1","doc":"Creates an expiration record from the provided values.\n\nAn expiration record contains properties defining expiration policies for a cache.\n\nA default value can be provided which will then be added as a default TTL to all keys\nwhich do not have one set explicitly. This must be a positive millisecond integer.\n\nThe interval being controlled here is the Janitor service schedule; it controls how\noften the purge runs in the background of your application to remove expired records.\nThis can be disabled completely by setting the value to nil. This is also a millisecond\ninteger.\n\nThe lazy value determines whether or not records can be lazily removed on read. Since\nthis is an expected behaviour it's enabled by default, but there are cases where you\nmight wish to disable it (such as when consistency isn't that big an issue).","ref":"Cachex.Spec.html#expiration/1"},{"type":"macro","title":"Cachex.Spec.expiration/2","doc":"Updates an expiration record from the provided values.","ref":"Cachex.Spec.html#expiration/2"},{"type":"macro","title":"Cachex.Spec.hook/1","doc":"Creates a hook record from the provided values.\n\nHook records contain the properties needed to start up a hook against a cache instance.\nThere are several components in a hook record:\n\n  * arguments to pass through to the hook init/1 callback.\n  * a flag to set whether or not a hook should fire asynchronously.\n  * the module name backing the hook, implementing the hook behaviour.\n  * options to pass to the hook server instance (allowing for names, etc).\n  * provisions to pass through to the hook provisioning callback.\n  * a PID reference to a potentially running hook instance (optional).\n  * the timeout to wait for a response when firing synchronous hooks.\n  * the type of the hook (whether to fire before/after a request).\n\nThese values are mainly provided by the user, and this record might actually be replaced\nin future with just a behaviour and a set of macros (as this record is very noisy now).","ref":"Cachex.Spec.html#hook/1"},{"type":"macro","title":"Cachex.Spec.hook/2","doc":"Updates a hook record from the provided values.","ref":"Cachex.Spec.html#hook/2"},{"type":"macro","title":"Cachex.Spec.hooks/1","doc":"Creates a hooks collection record from the provided values.\n\nHooks records are just a pre-sorted collection of hook records, grouped by their\ntype so that notifications internally do not have to iterate and group manually.","ref":"Cachex.Spec.html#hooks/1"},{"type":"macro","title":"Cachex.Spec.hooks/2","doc":"Updates a hooks record from the provided values.","ref":"Cachex.Spec.html#hooks/2"},{"type":"macro","title":"Cachex.Spec.router/1","doc":"Creates a router record from the provided values.\n\nA router record reprsents routing within a distributed cache. Each router record should have a\nvalid routing module provided, which correct implements the behaviour defined in `Cachex.Router`.\n\nOptions to be passed on router state initialization can also be provided, but note that all other\nvalues inside this structure are for internal use and will be overwritten as needed.","ref":"Cachex.Spec.html#router/1"},{"type":"macro","title":"Cachex.Spec.router/2","doc":"Updates a router record from the provided values.","ref":"Cachex.Spec.html#router/2"},{"type":"macro","title":"Cachex.Spec.warmer/1","doc":"Creates a warmer record from the provided values.\n\nA warmer record represents cache warmer processes to be run to populate keys.\n\nA warmer should have a valid module provided, which correctly implements the behaviour\nassociated with `Cachex.Warmer`. A state can also be provided, which will be passed\nto the execution callback of the provided module (which defaults to `nil`). The flag\n`:required` determines if the warmer much execute on cache startup.","ref":"Cachex.Spec.html#warmer/1"},{"type":"macro","title":"Cachex.Spec.warmer/2","doc":"Updates a warmer record from the provided values.","ref":"Cachex.Spec.html#warmer/2"},{"type":"type","title":"Cachex.Spec.cache/0","doc":"","ref":"Cachex.Spec.html#t:cache/0"},{"type":"type","title":"Cachex.Spec.command/0","doc":"","ref":"Cachex.Spec.html#t:command/0"},{"type":"type","title":"Cachex.Spec.entries/0","doc":"","ref":"Cachex.Spec.html#t:entries/0"},{"type":"type","title":"Cachex.Spec.entry/0","doc":"","ref":"Cachex.Spec.html#t:entry/0"},{"type":"type","title":"Cachex.Spec.expiration/0","doc":"","ref":"Cachex.Spec.html#t:expiration/0"},{"type":"type","title":"Cachex.Spec.hook/0","doc":"","ref":"Cachex.Spec.html#t:hook/0"},{"type":"type","title":"Cachex.Spec.hooks/0","doc":"","ref":"Cachex.Spec.html#t:hooks/0"},{"type":"type","title":"Cachex.Spec.router/0","doc":"","ref":"Cachex.Spec.html#t:router/0"},{"type":"type","title":"Cachex.Spec.warmer/0","doc":"","ref":"Cachex.Spec.html#t:warmer/0"},{"type":"module","title":"Cachex.Stats","doc":"Hook module to control the gathering of cache statistics.\n\nThis implementation of statistics tracking uses a hook to run asynchronously\nagainst a cache (so that it doesn't impact those who don't want it). It executes\nas a post hook and provides a solid example of what a hook can/should look like.\n\nThis hook has zero knowledge of the cache it belongs to; it keeps track of an\ninternal set of statistics based on the provided messages. This means that it\ncan also be mocked easily using raw server calls to `handle_notify/3`.","ref":"Cachex.Stats.html"},{"type":"function","title":"Cachex.Stats.for_cache/1","doc":"Retrieves the latest statistics for a cache.","ref":"Cachex.Stats.html#for_cache/1"},{"type":"behaviour","title":"Cachex.Warmer","doc":"Module controlling cache warmer behaviour definitions.\n\nThis module defines the cache warming implementation for Cachex, allowing the\nuser to register warmers against a cache to populate the tables periodically.\nDoing this allows for easy pulling against expensive values (such as those from\na backing database or remote server), without risking heavy usage.\n\nWarmers will block when the cache is started to ensure that the application\nwill not complete booting up, until your cache has been warmed. This guarantees\nthat there will not be any time where the application is available, without the\ndesired data in the cache.\n\nWarmers are fired on a schedule, and are exposed via a very simple behaviour of\njust a block to execute periodically.","ref":"Cachex.Warmer.html"},{"type":"callback","title":"Cachex.Warmer.execute/1","doc":"Executes actions to warm a cache instance.\n\nThis can either return values to set in the cache, or the atom `:ignore` to\nsignal that there's nothing to be set at this point in time. Values to be set\nshould be returned as `{ :ok, pairs }` where pairs is a list of `{ key, value }`\npairs to place into the cache via `Cachex.put_many/3`.\n\nIf you wish to provide expiration against the keys, you can return a Tuple in\nthe form `{ :ok, pairs, opts }` where `opts` is a list of options as accepted\nby the `Cachex.put_many/3` function, thus allowing you to expire your warmed\ndata. Unless this is provided, there is no explicit expiration associated with\nwarmed values (as predicting the appropriate expiration is not possible).\n\nThe argument provided here is the one provided as state to the warmer records\nat cache configuration time; it will be `nil` if none was provided.","ref":"Cachex.Warmer.html#c:execute/1"},{"type":"exception","title":"Cachex.Error","doc":"Module containing all error definitions used in the codebase.\n\nThis module allows users to catch Cachex errors in a separate block\nto other errors/exceptions rather than using stdlib errors:\n\n    iex> try do\n    ...>   Cachex.put!(:cache, \"key\", \"value\")\n    ...> rescue\n    ...>   e in Cachex.Error -> e\n    ...> end\n\nAll error messages (both shorthand and long form) can be found in this module,\nincluding the ability to convert from the short form to the long form using the\n`long_form/1` function.","ref":"Cachex.Error.html"},{"type":"macro","title":"Cachex.Error.error/1","doc":"Generates a tagged `:error` Tuple at compile time.\n\nThe provided error key must be contained in the list of known\nidentifiers returned be `known/0`, otherwise this call will fail.","ref":"Cachex.Error.html#error/1"},{"type":"function","title":"Cachex.Error.known/0","doc":"Returns the list of known error keys.","ref":"Cachex.Error.html#known/0"},{"type":"function","title":"Cachex.Error.long_form/1","doc":"Converts an error identifier to it's longer form.\n\nError identifiers should be atoms and should be contained in the\nlist of errors returned by `known/0`. The return type from this\nfunction will always be a binary.","ref":"Cachex.Error.html#long_form/1"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nCachex is an extremely fast in-memory key/value store with support for many useful features:\n\n- Time-based key expirations\n- Maximum size protection\n- Pre/post execution hooks\n- Proactive/reactive cache warming\n- Transactions and row locking\n- Asynchronous write operations\n- Distribution across app nodes\n- Syncing to a local filesystem\n- Idiomatic cache streaming\n- Batched write operations\n- User command invocation\n- Statistics gathering\n\nAll of these features are optional and are off by default so you can pick and choose those you wish to enable.","ref":"overview.html"},{"type":"extras","title":"Setting Up - Getting Started","doc":"To get started, please add Cachex to your `mix.exs` list of dependencies and then pull it using `mix deps.get`:\n\n```elixir\ndef deps do\n  [{:cachex, \"~> 4.0\"}]\nend\n```\n\nDepending on what you're trying to do, there are a couple of different ways you might want to go about starting a cache. If you're testing out Cachex inside `iex`, you can call `Cachex.start_link/2` manually:\n\n```elixir\nCachex.start_link(:my_cache)     # with default options\nCachex.start_link(:my_cache, []) # with custom options\n```\n\nIn other cases, you might want to start a cache within an existing supervision tree. If you created your project via `mix` with the `--sup` flag, this should be available to you inside `lib/my_app/application.ex`:\n\n```elixir\nchildren = [\n  {Cachex, [:my_cache]},     # with default options\n  {Cachex, [:my_cache, []]}  # with custom options\n]\n```\n\nBoth of these approaches work the same way; your options are parsed and your cache is started as a child under the appropriate parent process. The latter is recommended for production applications, as it will ensure your cache is managed correctly inside your application.","ref":"overview.html#setting-up"},{"type":"extras","title":"Basic Examples - Getting Started","doc":"Working with a cache is pretty straightforward, and basically everything is provided by the core `Cachex` module. You can make calls to a cache using the name you registered it under at startup time.\n\nLet's take a quick look at some basic calls you can make to a cache in a quick `iex` session:\n\n```elixir\n# create a default cache in our shell session\n{:ok, _pid} = Cachex.start_link(:my_cache)\n\n# place a \"my_value\" string against the key \"my_key\"\n{:ok, true} = Cachex.put(:my_cache, \"my_key\", \"my_value\")\n\n# verify that the key exists under the key name\n{:ok, true} = Cachex.exists?(:my_cache, \"my_key\")\n\n# verify that \"my_value\" is returned when we retrieve\n{:ok, \"my_value\"} = Cachex.get(:my_cache, \"my_key\")\n\n# remove the \"my_key\" key from the cache\n{:ok, true} = Cachex.del(:my_cache, \"my_key\")\n\n# verify that the key no longer exists\n{:ok, false} = Cachex.exists?(:my_cache, \"my_key\")\n\n# verify that \"my_value\" is no longer returned\n{:ok, nil} = Cachex.get(:my_cache, \"my_key\")\n```\n\nIt's worth noting here that the actions supported by the Cachex API have an automatically generated \"unsafe\" equivalent (i.e. appended with `!`). These options will unpack the returned tuple, and return values directly:\n\n```elixir\n# calling by default will return a tuple\n{:ok, nil} = Cachex.get(:my_cache, \"key\")\n\n# calling with `!` unpacks the tuple\nnil = Cachex.get!(:my_cache, \"key\")\n\n# causing an error will return an error tuple value\n{:error, :no_cache} =  Cachex.get(:missing_cache, \"key\")\n\n# but calling with `!` raises the error\nCachex.get!(:missing_cache, \"key\")\n** (Cachex.Error) Specified cache not running\n    (cachex) lib/cachex.ex:249: Cachex.get!/3\n```\n\nThe `!` version of functions exists for convenience, in particular to make chaining and assertions easier in unit testing. For production use cases it's recommended to avoid `!` wrappers, and instead explicitly handle the different response types.","ref":"overview.html#basic-examples"},{"type":"extras","title":"Advanced Examples - Getting Started","doc":"Beyond the typical get/set semantics of a cache, Cachex offers many additional features to help with typical use cases and access patterns a developer may meeting during their day-to-day.\n\nWhile the list is too long to properly cover everything in detail here, let's take a look at some of the most common cache actions:\n\n```elixir\n# create a default cache in our shell session\n{:ok, _pid} = Cachex.start_link(:my_cache)\n\n# place some values in a single batch call\n{:ok, true} = Cachex.put_many(:my_cache, [\n    {\"key1\", 1},\n    {\"key2\", 2},\n    {\"key3\", 3}\n])\n\n# now let's do an atomic update operation against the key in the cache\n{:commit, 2} = Cachex.get_and_update(:my_cache, \"key1\", fn value ->\n    value + 1\nend)\n\n# we can also do this via `Cachex.incr/2`\n{:ok, 2} = Cachex.incr(:my_cache, \"key2\")\n\n# and of course the inverse via `Cachex.decr/2`\n{:ok, 0} = Cachex.decr(:my_cache, \"key3\")\n\n# we can also lazily compute keys if they're missing from the cache\n{:commit, \"nazrat\"} = Cachex.fetch(:my_cache, \"tarzan\", fn key ->\n  {:commit, String.reverse(key)}\nend)\n\n# we can also write keys with an time expiration (in milliseconds)\n{:ok, true} = Cachex.put(:my_cache, \"secret_mission\", \"...\", expire: 1)\n\n# and if we pull it back after expiration, it's not there!\n{:ok, nil} = Cachex.get(:my_cache, \"secret_mission\")\n```\n\nThese are just some of the conveniences made available by Cachex's API, but there's a still a bunch of other fun stuff in the `Cachex` API, covering a broad range of patterns and use cases.\n\nFor further information or examples on supported features and options, please see the Cachex [documentation](https://hexdocs.pm/cachex) where there are several guides on specific features and workflows.\n\nAll of the hosted documentation is also available in raw form in the [repository](https://github.com/whitfin/cachex/tree/main/docs).","ref":"overview.html#advanced-examples"},{"type":"extras","title":"Batching Actions","doc":"# Batching Actions\n\nIt's sometimes the case that you need to execute several cache actions in a row. Although you can do this in the normal, this is actually somewhat inefficient as each call has to do various management (such as looking up cache states). For this reason Cachex offers several mechanisms for making multiple calls in sequence.","ref":"batching-actions.html"},{"type":"extras","title":"Submitting Batches - Batching Actions","doc":"The simplest way to make several cache calls together is `Cachex.execute/3`. This API allows the caller to provide a function which will be provided with a pre-validated cache state which can be used (instead of the cache name) to execute cache actions. This will skip all of the cache management overhead you'd see typically:\n\n```elixir\n# standard way to execute several actions\nr1 = Cachex.get!(:my_cache, \"key1\")\nr2 = Cachex.get!(:my_cache, \"key2\")\nr3 = Cachex.get!(:my_cache, \"key3\")\n\n# using Cachex.execute/3 to optimize the batch of calls\n{r1, r2, r3} =\n  Cachex.execute!(:my_cache, fn cache ->\n    # execute our batch of actions\n    r1 = Cachex.get!(cache, \"key1\")\n    r2 = Cachex.get!(cache, \"key2\")\n    r3 = Cachex.get!(cache, \"key3\")\n\n    # pass back all results as a tuple\n    {r1, r2, r3}\n  end)\n```\n\nAlthough this syntax might look a little more complicated at a glance, it should be fairly straightforward to get used to. The small change in approach here gives a fairly large boost to cache throughput. To compare the two examples above, we can use a tool like [Benchee](https://github.com/PragTob/benchee) for a rough comparison:\n\n```\nName                   ips        average  deviation         median         99th %\ngrouped             1.72 M      580.68 ns  ±3649.68%         500 ns         750 ns\nindividually        1.31 M      764.02 ns  ±2335.25%         625 ns         958 ns\n```\n\nWe can clearly see the time saving when using the batched approach, even if there is a large deviation in the numbers above. Somewhat intuitively, the time saving scales to the number of actions you're executing in your batch, even if it is unlikely that anyone is doing more than a few calls at once.\n\nIt's important to note that even though you're executing a batch of actions, other processes can access and modify keys at any time during your `Cachex.execute/3` call. These calls still occur your calling process; they're not sent through any kind of arbitration process. To demonstrate this, here's a quick example:\n\n```elixir\n# start our execution block\nCachex.execute!(:my_cache, fn cache ->\n  # set a base value in the cache\n  Cachex.put!(cache, \"key\", \"value\")\n\n  # we're paused but other changes can happen\n  :timer.sleep(5000)\n\n  # this may have have been set elsewhere\n  Cachex.get!(cache, \"key\")\nend)\n```\n\nAs we wait 5 seconds before reading the value back, the value may have been modified or even removed by other processes using the cache (such as TTL cleanup or other places in your application). If you want to guarantee that nothing is modified between your interactions, you should consider a transactional block instead.","ref":"batching-actions.html#submitting-batches"},{"type":"extras","title":"Transactional Batches - Batching Actions","doc":"A transactional block will guarantee that your actions against a cache key will happen with zero interaction from other processes. Transactions look almost exactly the same as `Cachex.execute/3`, except that they require a list of keys to lock for the duration of their execution.\n\nThe entry point to a Cachex transaction is (unsurprisingly) `Cachex.transaction/4`. If we take the example from the previous section, let's look at how we can guarantee consistency between our cache calls:\n\n```elixir\n# start our execution block\nCachex.transaction!(:my_cache, [\"key\"], fn cache ->\n  # set a base value in the cache\n  Cachex.put!(cache, \"key\", \"value\")\n\n  # we're paused but other changes will not happen\n  :timer.sleep(5000)\n\n  # this will be guaranteed to return \"value\"\n  Cachex.get!(cache, \"key\")\nend)\n```\n\nIt's critical to provide the keys you wish to lock when calling `Cachex.transaction/4`, as any keys not specified will still be available to be written by other processes during your function's execution. If you're making a simple cache call, the transactional flow will only be taken if there is a simultaneous transaction happening against the same key. This enables caches to stay lightweight whilst allowing for these batches when they really matter.\n\nAnother pattern which may prove useful is providing an empty list of keys, which will guarantee that your transaction runs at a time when no keys in the cache are currently locked. For example, the following code will guarantee that no keys are locked when purging expired records:\n\n```elixir\nCachex.transaction!(:my_cache, [], fn cache ->\n  Cachex.purge!(cache)\nend)\n```\n\nTransactional flows are only enabled the first time you call `Cachex.transaction/4`, so you shouldn't see any peformance penalty in the case you're not actively using transactions. This also has the benefit of not requiring transaction support to be configured inside the cache options, as was the case in earlier versions of Cachex.\n\nThe last major difference between `Cachex.execute/3` and `Cachex.transaction/4` is where they run; transactions are executed inside a secondary worker process, so each transaction will run only after the previous has completed. As such there is a minor performance overhead when working with transactions, so use them only when you need to.","ref":"batching-actions.html#transactional-batches"},{"type":"extras","title":"Local Persistence","doc":"# Local Persistence\n\nCachex ships with basic support for saving a cache to a local file using the [External Term Format](https://www.erlang.org/doc/apps/erts/erl_ext_dist). These files can then be used to seed data into a new instance of a cache to persist values between cache instances. As it stands all persistence must be handled manually via the Cachex API, although additional features may be added in future to add convenience around this.","ref":"local-persistence.html"},{"type":"extras","title":"Writing to Disk - Local Persistence","doc":"To save a cache to a file on disk, you can use the `Cachex.save/3` function. This function will handle compression automatically and populate the path on disk with a file you can import later. It should be noted that the internal format of this file should not be relied upon.\n\n```elixir\n{ :ok, true } = Cachex.save(:my_cache, \"/tmp/my_cache.dat\")\n```\n\nThe above demonstrates how simple it is to save your cache to a location on disk (in this case `/tmp/my_cache.dat`). Any options can be provided as a `Keyword` list as an optional third parameter.","ref":"local-persistence.html#writing-to-disk"},{"type":"extras","title":"Loading from Disk - Local Persistence","doc":"To seed a cache from an existing file, you can use `Cachex.restore/3`. This will *merge* the file into your cache, overwriting and clashing keys and maintaining any keys which existed in the cache beforehand. If you want a direct match of the file inside your cache, you should use `Cachex.clear/2` before loading your data.\n\n```elixir\n# optionally clean your cache first\n{ :ok, _amt } = Cachex.clear(:my_cache)\n\n# then you can load the existing save into your cache\n{ :ok, true } = Cachex.restore(:my_cache, \"/tmp/my_cache.dat\")\n```\n\nPlease note that loading from an existing file will maintain all existing expirations, and records which have already expired will *not* be added to the cache table. This should not be surprising, but it is worth calling out.","ref":"local-persistence.html#loading-from-disk"},{"type":"extras","title":"Streaming Records","doc":"# Streaming Records\n\nCachex provides the ability to create an Elixir `Stream` seeded by the contents of a cache, using an ETS table continuation and `Stream.resource/3`. This then allows the developer to use any of the `Enum` or `Stream` module functions against the entries in cache, which can be a very powerful and flexible tool.","ref":"streaming-records.html"},{"type":"extras","title":"Basic Streams - Streaming Records","doc":"By default, `Cachex.stream/3` will return a `Stream` over all entries in a cache which are yet to expire (at the time of stream creation). These cache entries will be streamed as `Cachex.Spec.entry` records, so you can use pattern matching to pull any of the entry fields assuming you have `Cachex.Spec` imported:\n\n```elixir\n# store some values in the cache\nCachex.start(:my_cache)\nCachex.put(:my_cache, \"one\", 1)\nCachex.put(:my_cache, \"two\", 2)\nCachex.put(:my_cache, \"three\", 3)\n\n# create our cache stream of all records\n{ :ok, stream } = Cachex.stream(:my_cache)\n\n# sum up all the cache record values, which == 6\nEnum.reduce(stream, 0, fn entry(value: value), total ->\n  total + value\nend)\n```","ref":"streaming-records.html#basic-streams"},{"type":"extras","title":"Efficient Querying - Streaming Records","doc":"While the `Enum` module provides the ability to filter records easily, we can do better by pre-filtering using a match specification. Under the hood these matches are as defined by the Erlang documentation, and can be passed as the second argument to `Cachex.stream/3`.\n\nTo avoid having to handle Cachex implementation details directly, the `Cachex.Query` module exposes a few functions designed to assist with creation of new queries. If we take our example above, we can use a query to sum only the odd numbers in the table without having to filter on the Elixir side:\n\n```elixir\n# for matching\nimport Cachex.Spec\n\n# store some values in the cache\nCachex.start(:my_cache)\nCachex.put(:my_cache, \"one\", 1)\nCachex.put(:my_cache, \"two\", 2)\nCachex.put(:my_cache, \"three\", 3)\n\n# generate our filter to find odd values\nfilter = {:==, {:rem, :value, 2}, 1}\n\n# generate the query using the filter, only return `:value\nquery = Cachex.Query.build(where: filter, output: :value)\n\n# == 4\n:my_cache\n|> Cachex.stream!(query)\n|> Enum.sum()\n```\n\nRather than retrieve and handle the whole cache entry, here we're using `:output` to choose only the `:value` column from each entry. This lets us skip out on `Enum.reduce/3` and go directly to `Enum.sum/1`, much easier!\n\nIt's important to note here is that cache queries do *not* distinguish between expired records in a cache; they match across all records within a cache. This is a change in Cachex v4.x to provide more flexibility in other areas of the Cachex library. If you want to filter out expired records, you can use the `Cachex.Query.expired/1` convenience function:\n\n```elixir\n# store some values in the cache\nCachex.start(:my_cache)\nCachex.put(:my_cache, \"one\", 1)\nCachex.put(:my_cache, \"two\", 2)\nCachex.put(:my_cache, \"three\", 3)\n\n# generate our filter to find odd values\nfilter = {:==, {:rem, :value, 2}, 1}\n\n# wrap our filter to filter expired values\nfilter = Cachex.Query.expired(filter)\n\n# generate the query using the filter, only return `:value\nquery = Cachex.Query.build(where: filter, output: :value)\n\n# == 4\n:my_cache\n|> Cachex.stream!(query)\n|> Enum.sum()\n```\n\nThis function accepts a query guard and wraps it with clauses to filter out expired records. The returned guard can then be passed to `Cachex.Query.build/1` to return only the expired records which match your query. This is all fairly simple, but it's definitely something to keep in mind when working with `Cachex.Query`!","ref":"streaming-records.html#efficient-querying"},{"type":"extras","title":"Limiting Caches","doc":"# Limiting Caches\n\nCache limits are restrictions on a cache to ensure that it stays within given bounds. The limits currently shipped inside Cachex are based around the number of entries inside a cache, but there are plans to add new policies in future (for example basing the limits on memory spaces). You even even write your own!","ref":"limiting-caches.html"},{"type":"extras","title":"Manual Pruning - Limiting Caches","doc":"The main entrypoint to cache limitation included in Cachex is `Cachex.prune/3`, which provides a Least Recently Written (LRW) implementation of pruning a cache. This means that we calculate the oldest entries, and remove them from the cache in order to make room for new entries. It should be noted that \"oldest\" in this context means \"those written or updated longest ago\".\n\nYou can trigger a pruning manually via `Cachex.prune/3`, passing the maximum size of a cache to shrink to:\n\n```elixir\n# start a new cache\nCachex.start(:my_cache)\n\n# insert 100 keys\nfor i <- 1..100 do\n  Cachex.put!(:my_cache, i, i)\nend\n\n# guarantee we have 100 keys in the cache\n{ :ok,  100 } = Cachex.size(:my_cache)\n\n# trigger a pruning down to 50 keys only\n{ :ok, true } = Cachex.prune(:my_cache, 50, reclaim: 0)\n\n# verify that we're down to 50 keys\n{ :ok,   50 } = Cachex.size(:my_cache)\n```\n\nThe `:reclaim` option can be used to reduce thrashing, by evicting an additional number of entries. In the case above the next write would cause the cache to once again need pruning, and then so on. The `:reclaim` option accepts a percentage (as a decimal) of extra keys to evict, which gives us a buffer between pruning of a cache.\n\nTo demonstrate this we can run the same example as above, except using a `:reclaim` of `0.1` (the default). This time we'll be left with 45 keys instead of 50, as we reclaimed an extra 10% of the table (`50 * 0.1 = 5`):\n\n```elixir\n# start a new cache\nCachex.start(:my_cache)\n\n# insert 100 keys\nfor i <- 1..100 do\n  Cachex.put!(:my_cache, i, i)\nend\n\n# guarantee we have 100 keys in the cache\n{ :ok,  100 } = Cachex.size(:my_cache)\n\n# trigger a pruning down to 50 keys, reclaiming 10%\n{ :ok, true } = Cachex.prune(:my_cache, 50, reclaim: 0.1)\n\n# verify that we're down to 45 keys\n{ :ok,   45 } = Cachex.size(:my_cache)\n```\n\nIt is almost never a good idea to set `reclaim: 0` unless you have very specific use cases, so if you don't it's recommended to leave `:reclaim` at the default value - it was only used above for example purposes.","ref":"limiting-caches.html#manual-pruning"},{"type":"extras","title":"Lifecycle Pruning - Limiting Caches","doc":"Although you can manually prune a cache, in reality this isn't particularly useful as you want to be able to continually monitor a cache's size. For this reason, Cachex includes several lifecycle hooks to trigger `Cachex.Limit` automatically. This will give you a monitored cache size, easily configured at cache startup:\n\n```elixir\n# include records\nimport Cachex.Spec\n\n# maximum 500 entries, LRW eviction, default trim\nCachex.start(:my_cache,\n  hooks: [\n    hook(module: Cachex.Limit.Scheduled, args: {\n      500,  # setting cache max size\n      [],   # options for `Cachex.prune/3`\n      []    # options for `Cachex.Limit.Scheduled`\n    })\n  ]\n)\n```\n\nThis will spawn a cache hook to continually prune your cache periodically, based on the options you provided. You can pass options for `Cachex.prune/3` as the second element of the `args` tuple, and customize the hook itself in the third element. Currently the only supported parameter for the hook is `:frequency`, which defaults to `1000` (one second). Please see the documentation for an updated list of supported configuration.\n\nIf you need more exact timing you can opt to use `Cachex.Limit.Evented` rather than `Cachex.Limit.Scheduled`, which will react to hook events inside a cache instead of running on a schedule:\n\n```elixir\n# include records\nimport Cachex.Spec\n\n# maximum 500 entries, LRW eviction, default trim\nCachex.start(:my_cache,\n  hooks: [\n    hook(module: Cachex.Limit.Evented, args: {\n      500,  # setting cache max size\n      []    # options for `Cachex.prune/3`\n    })\n  ]\n)\n```\n\nThis is a much more accurate policy, but has a much higher memory and CPU overhead due to hooking into the main lifecycle events. If you can, it's recommended to use the scheduling approach for this reason.\n\nIt should be hopefully be evident from the above, but lifecycle pruning is not instant - in general it is extremely quick, however if you have a cache limit of 500 keys and you add 500,000 keys, the cleanup does take a few hundred milliseconds to occur (that's a lot to clean). This shouldn't affect most users, but it is something to point out and be aware of.","ref":"limiting-caches.html#lifecycle-pruning"},{"type":"extras","title":"LRU Style Pruning - Limiting Caches","doc":"In addition to the two lifecycle controls for LRW based caching, Cachex v4.x also includes a naive solution for those who wish to use Least Recently Used (LRU) based approaches via `Cachex.Limit.Accessed`.\n\nThis is done as an extension of LRW caching by attaching a small lifecycle hook to update the access time of each record, thus making the existing LRW hooks able to handle them as usual. Due to this you can freely choose either of the two LRW approaches above, by placing the LRW access hook *before* it in the cache initialization:\n\n```elixir\n# include records\nimport Cachex.Spec\n\n# maximum 500 entries, LRW eviction, default trim\nCachex.start(:my_cache,\n  hooks: [\n    hook(module: Cachex.Limit.Accessed),\n    hook(module: Cachex.Limit.Scheduled, args: {\n      500,  # setting cache max size\n      [],   # options for `Cachex.prune/3`\n      []    # options for `Cachex.Limit.Scheduled`\n    })\n  ]\n)\n```\n\nAs you might expect, there is a fair cost to this as `Cachex.Limit.Accessed` must listen to and act on cache events when a key is accessed. This can result in heavy read/write activity within a cache. For this reason it's recommended to operate using LRW when possible, and LRU only as absolutely necessary.","ref":"limiting-caches.html#lru-style-pruning"},{"type":"extras","title":"Expiring Records","doc":"# Expiring Records\n\nCachex implements several different ways to work with key expiration, with each operating with slightly different behaviour. The two main techniques in use currently are background expiration and lazy expiration. Although there are cases where you may wish to only use one of these approaches, you'll generally want a combination of both to ensure correctness of your cache. By default Cachex will combine both approaches to provide more intuitive behaviour for the developer.","ref":"expiring-records.html"},{"type":"extras","title":"Janitor Services - Expiring Records","doc":"The Cachex Janitor is a background process used to purge the internal cache tables periodically. The Janitor operates using a full table sweep of records to ensure consistency and correctness. As such, a Janitor sweep will run somewhat less frequently - by default only once every few seconds. This frequency can be controlled by the developer, and can be controlled on a per-cache basis.\n\nIn the current version of Cachex, the Janitor is pretty well optimized as most of the work happens in the ETS layer. As a rough benchmark, it can check and purge 500,000 expired records in around a second (where the removal is a majority of the work). Keep in mind that the frequency of the Janitor execution has an impact on the memory being held by the expired keyset in your cache. For most use cases the default frequency should be just fine. If you need to, you can customize the frequency on which the Janitor runs:\n\n```elixir\nimport Cachex.Spec\n\nCachex.start(:my_cache, [\n    expiration: expiration(interval: :timer.seconds(3))\n])\n```\n\nThe Janitor is the only feature which is enabled by default, as it was misleading for users when it was not running by default. To disable the Janitor completely, you can set the `:interval` option to `nil`. In this case you will either be fully reliant on lazy expirations, or have to implement your own expiration handling.\n\nPlease note that this is rolling interval that is set to trigger after completion of a run, meaning that if you schedule a Janitor every 5s it will be 5s after a successful run rather than 5s after the last trigger fired to start a run.","ref":"expiring-records.html#janitor-services"},{"type":"extras","title":"Lazily Expiring Keys - Expiring Records","doc":"A cache record contains an internal modification time, as well as an associated expiration time. These values do not change unless explicitly modified by a cache call. This means that we have access to these values when fetching a key, which allows us to quickly check expirations on retrieval.\n\nIf a key is retrieved after the expiration has passed, the key will be removed at that time and return `nil` to the caller just as if the key did not exist in the cache. This provides guarantees of consistency even if the Janitor hasn't run recently; you can still never accidentally fetch an expired key. In turn this allows us to run the Janitor a little less frequently as we don't have to be scared of stale values.\n\nThere is a very minimal overhead to this lazy checking, and there are cases where you don't need to be as accurate. For these reasons you can easily disable this behaviour by seting the `:lazy` option to false at cache startup:\n\n```elixir\nimport Cachex.Spec\n\nCachex.start(:my_cache, [\n    expiration: expiration(lazy: false)\n])\n```\n\nAnother advantage of disabling this checking is that the execution times of your read operations become more uniform; there's no longer a case where a deletion may make a read take a little longer. That being said, the overhead is so small that it's recommended to leave this enabled unless you absolutely know you don't need it.\n\nNaturally this technique cannot stand alone as it only evicts on key retrieval; if you never touch a record again, it would never be expired and thus your cache would just keep growing. For this reason the Janitor is enabled by default when an expiration is set to protect the user from memory errors in their application. It should also be noted that this approach only applies to single key retrieval; it does not activate on batch reads (such as `Cachex.stream/3`).","ref":"expiring-records.html#lazily-expiring-keys"},{"type":"extras","title":"Providing Key Expirations - Expiring Records","doc":"There are a number of ways to provide expirations for entries inside a cache:\n\n* Setting a default expiration for a cahe via `Cachex.start_link/2`\n* Setting an expiration manually via `Cachex.expire/4` or `Cachex.expire_at/4`\n* Setting the `:expire` option within calls to `Cachex.put/4` or `Cachex.put_many/3`\n* Setting the `:expire` option within return tuples in `Cachex.fetch/4` or `Cachex.get_and_update/4`\n\nEach of these approches is handled the same way internally, they just provide sugar for various use cases. In general you should visit the appropriate functions for the documentation of how to use them, but here are some examples:\n\n\n```elixir\nimport Cachex.Spec\n\n# default for all entries\nCachex.start(:my_cache, [\n    expiration: expiration(default: :timer.seconds(60))\n])\n\n# setting an expiration manually\nCachex.put(:my_cache, \"key\", \"value\")\nCachex.expire(:my_cache, \"key\", :timer.seconds(60))\n\n# using the `Cachex.put/4` shorthand rather than setting manually\nCachex.put(:my_cache, \"key\", \"value\", expire: :timer.seconds(60))\n\n# setting expiration on lazily computed values\nCachex.fetch(:my_cache, \"key\", fn ->\n    { :commit, \"value\", expire: :timer.seconds(60) }\nend)\n```\n\nThere is no strong recommendation as to which you use, most of it falls to developer preference. The overhead of setting expirations is quite minimal, so feel free to take your pick. If you want the absolute fastest, inlining the `:expire` option against `Cachex.put/4` will be your best option.","ref":"expiring-records.html#providing-key-expirations"},{"type":"extras","title":"Gathering Statistics","doc":"# Gathering Statistics\n\nCachex includes basic support for tracking statistics in a cache, so you can look at things like throughput and hit/miss rates. This is provided via the `Cachex.Stats` hook implementation.","ref":"stats-gathering.html"},{"type":"extras","title":"Configuration - Gathering Statistics","doc":"As of Cachex v4.x this is configured as a hook during cache initialization:\n\n```elixir\n# include records\nimport Cachex.Spec\n\n# create a cache with stats\nCachex.start(:my_cache,\n  hooks: [\n    hook(module: Cachex.Stats)\n  ]\n)\n\n# insert 100 keys\nfor i <- 1..100 do\n  Cachex.put!(:my_cache, i, i)\nend\n\n# generate both a cache hit and a miss\n{ :ok,   1 } = Cachex.get(:my_cache, 1)\n{ :ok, nil } = Cachex.get(:my_cache, 101)\n\n# print stats\n:my_cache\n|> Cachex.stats!()\n|> IO.inspect\n```\n\nRunning this will give you a map of various statistics based on the actions and operations taken by your cache.","ref":"stats-gathering.html#configuration"},{"type":"extras","title":"Example Statistics - Gathering Statistics","doc":"The statistics map returned by `Cachex.stats/2` should look something like the example below (at the time of writing):\n\n```elixir\n%{\n  meta: %{creation_date: 1726777631670},\n  hits: 1,\n  misses: 1,\n  hit_rate: 50.0,\n  miss_rate: 50.0,\n  calls: %{get: 2, put: 100},\n  operations: 102,\n  writes: 100\n}\n```\n\nAs you can see, we see the breakdown of calls to the cache, the hit/miss rate, the total writes to a cache, etc. This is useful when gauging how much time your cache is actually saving and allows you to determine that everything is working as intended.\n\nIt should be noted that the output format of `Cachex.stats/2` is *not* considered part of the Public API for backwards compatibility; the shape of this may change as and when it's necessary to do so.","ref":"stats-gathering.html#example-statistics"},{"type":"extras","title":"Cache Routers","doc":"# Cache Routers\n\nNew in Cachex v4.x, routing provides the developer the ability to determine how keys are assigned to nodes in a distributed caching cluster.\n\nIn previous versions of Cachex (namely v3.x) although there was support for routing within a cluster, the routing algorithm was neither configurable nor flexible. This lead to scenarios where it was simply insufficient, such as dynamically scaling caches. The new `Cachex.Router` module hopes to provide more flexibility to the developer, enabling them to choose the routing algorithm which best fits their use case.","ref":"cache-routers.html"},{"type":"extras","title":"Default Routers - Cache Routers","doc":"Cachex ships with several routers included, in an attempt to handle the most common use cases easily. The current set of included routers is as follows (at the time of writing):\n\n| Module                | Description                                                                      |\n|-----------------------|----------------------------------------------------------------------------------|\n| `Cachex.Router.Local` | Routes keys to the local node only (the default)                                 |\n| `Cachex.Router.Mod`   | Routes keys to a node using basic modulo hashing (i.e. `hash(key) % len(nodes)`) |\n| `Cachex.Router.Jump`  | Routes keys to a node using the Jump Consistent hash algorithm                   |\n| `Cachex.Router.Ring`  | Routes keys to a node using Discord's hash ring implementation                   |\n\nEach of these routers has different strengths and weaknesses, so it's up to you to choose which best fits your use case. As a rule of thumb:\n\n* If you are using a single node, use `Cachex.Router.Local`\n* If you are using a statically sized cluster, use `Cachex.Router.Mod` or `Cachex.Router.Jump`\n* If you are using a dynamically sized cluster, use `Cachex.Router.Ring`\n* If you want the same behaviour as Cachex v3.x, use `Cachex.Router.Jump`\n\nOnce you know which router you want, you can configure it in your cache's options.","ref":"cache-routers.html#default-routers"},{"type":"extras","title":"Selecting a Router - Cache Routers","doc":"To select a router for your cache, you should provide the `:router` option when starting your cache:\n\n```elixir\n# for records\nimport Cachex.Spec\n\n# create a cache with a router\nCachex.start(:my_cache, [\n  router: router(module: Cachex.Router.Local)\n])\n```\n\nYou can also provide options to pass to the router during initialization, in the case your router supports different configurations:\n\n```elixir\n# for records\nimport Cachex.Spec\n\n# create a cache with a router\nCachex.start(:my_cache, [\n  router: router(\n    module: Cachex.Router.Jump,\n    options: [\n      nodes: [self()]\n    ]\n  )\n])\n```\n\nPlease see the module documentation for each router for further information, including options which may be used to customize the behaviour of the router.","ref":"cache-routers.html#selecting-a-router"},{"type":"extras","title":"Implementing Routers - Cache Routers","doc":"Although Cachex's included routers should be sufficient for many cases, they likely won't be enough for every case. For this reason it's possible for a developer to write their own router to have more control over a cache.\n\nA router is defined by the behaviour `Cachex.Router`. Implementing this behaviour in your own router will allow you to provide it as a module to the `:router` option at cache startup, and Cachex will automatically plug into it when routing keys in a cluster. The behaviour looks something like this:\n\n```elixir\n@doc \"\"\"\nInitialize a routing state for a cache.\n\"\"\"\n@callback init(cache :: Cachex.t(), options :: Keyword.t()) :: any\n\n@doc \"\"\"\nRetrieve the list of nodes from a routing state.\n\"\"\"\n@callback nodes(state :: any) :: [atom]\n\n@doc \"\"\"\nRoute a key to a node in a routing state.\n\"\"\"\n@callback route(state :: any, key :: any) :: atom\n\n@doc \"\"\"\nCreate a child specification to back a routing state.\n\"\"\"\n@callback children(cache :: Cachex.t(), options :: Keyword.t()) ::  Supervisor.child_spec()\n```\n\nAs a demonstration let's walk through implementing a router using the logic that [Redis](https://redis.io/docs/latest/operate/oss_and_stack/management/scaling/) follows.\n\nAt the time of writing Redis will generate a CRC16 for a key, and then route it to one of 16384 hash slots distributed around a cluster. Hash slots are assigned in groups, so a 3 node cluster would look like this:\n\n* Node A contains hash slots from 0 to 5500.\n* Node B contains hash slots from 5501 to 11000.\n* Node C contains hash slots from 11001 to 16383.\n\nUsing this information, we can create a `Cachex.Router` implementation to do something similar. We'll use the [crc](https://hexdocs.pm/crc) package to generate our CRC16 values:\n\n```elixir\ndefmodule MyCustomRouter do\n  @moduledoc \"\"\"\n  A very simple demonstration router based on Redis.\n  \"\"\"\n  use Cachex.Router\n\n  # our available slots\n  @max_slots 16384\n\n  @doc \"\"\"\n  Initialize the router state.\n\n  This will return a list of connected nodes in our cluster.\n  \"\"\"\n  def init(_cache, _options),\n    do: [node() | :erlang.nodes(:connected)]\n\n  @doc \"\"\"\n  Retrieve the nodes in our router state.\n\n  As our state is just a list of nodes, this is returned as-is.\n  \"\"\"\n  def nodes(nodes),\n    do: nodes\n\n  @doc \"\"\"\n  Routes a key to a node in the router state.\n\n  This will implement our main logic, returning the name of a\n  node that the provided key should be routed over to.\n  \"\"\"\n  def route(nodes, key) do\n    # generate our CRC16 value\n    crc_for_key = CRC.crc_16(key)\n\n    # calculate the number of slots per node\n    slots_per_node = trunc(16384 / length(nodes))\n\n    # create groups of slots to compare with\n    slots_for_nodes =\n      0..(@max_slots - 1)\n      |> Enum.chunk_every(slots_per_node)\n      |> Enum.with_index()\n\n    # convert our CRC16 to a slot in the cluster\n    slot_for_key = rem(crc_for_key, @max_slots)\n\n    # locate the group which contains our slot\n    {_group, idx} =\n      Enum.find(slots_for_nodes, fn {slots, _idx} ->\n        Enum.member?(slots, slot_for_key)\n      end)\n\n    # return the node name\n    Enum.at(nodes, idx)\n  end\nend\n```\n\nThis is obviously a very naive implementation for demonstration purposes; it could definitely be improved. That being said, hopefully this shows how easy it is to create our own router for our own requirements.\n\nYou may have noticed that we didn't need to implement `children/2`; this is because we can determine our key routing without need of any child processes. More complicated routers are able to spawn child processes under the main cache supervision tree in order to handle more complicated operations which require extra state management.\n\nWith our completed router, we can now create a cache and pass it in at startup:\n\n```elixir\n# for records\nimport Cachex.Spec\n\n# create a cache with our router\nCachex.start(:my_cache, [\n  router: router(module: MyCustomRouter)\n])\n```\n\nRouting will now be managed by our custom routing logic, instead of the default Cachex router; we no longer have to rely on routing implementations to be included alongside Cachex!","ref":"cache-routers.html#implementing-routers"},{"type":"extras","title":"Distributed Caches","doc":"# Distributed Caches\n\nA distributed cache is a cache spanning multiple nodes which allows each individual node to store less data in memory, while maintaining the ability to access data stored on other nodes. This is all pretty transparent; in general you shouldn't have to think too much about it.\n\nTo demonstrate how this works, let's walk through a simple scenario:\n\n* Let's say you have a cluster of 3 nodes\n* You write 100 keys to a cache in your application\n* You can expect e.g. ~30 keys stored on each node\n* Writing a key on Node A may actually store the value on Node B\n* Searching for that key on Node C will know to fetch it from Node B\n\nThe idea is that your cache is readily warm on each node in your cluster, even if the call to populate the key didn't occur on that node. If someone does an action when connected to Node A, it's placed in a hot layer accessible by both Node B and Node C even though no action was taken on either of those nodes.\n\nIt is important to be aware that the data is **not** replicated to every node; Cachex is a caching library, not a database. A cache provides an ephemeral data layer for an application; if you need data persistence across a cluster, you should another tool.\n\nFor further information on why Cachex should not be treated as a database, please see the additional context provided in [the related issue](https://github.com/whitfin/cachex/issues/246#issuecomment-2045591509).","ref":"distributed-caches.html"},{"type":"extras","title":"Cluster Routers - Distributed Caches","doc":"Distributed caches are controleld by the `Cachex.Router` implementation being used by a cache. The default router will only use the local node for key storage/retrieval, so we have to select a more appropriate router.\n\nCachex v4.x includes a new router based on Discord's [ex_hash_ring](https://github.com/discord/ex_hash_ring) library; this is a good router to get started with when using a distributed cache. It supports addition and removal of nodes based on OTP events, allowing for common use cases like Docker and Kubernetes.\n\nTo use this router at startup, provide the `:router` option when you call `Cachex.start_link/2`:\n\n```elixir\n# for records\nimport Cachex.Spec\n\n# create a cache with a ring router\nCachex.start(:my_cache, [\n  router: router(module: Cachex.Router.Ring)\n])\n```\n\nIf you wish to customize the behaviour of the router, you can see the supported options at `Cachex.Router.Ring.init/2`. These options can be provided in the same `router` record under the `:options` key.\n\nFor example to create a ring router which listens for addition/removal of nodes, we can set the `:monitor` option:\n\n```elixir\n# for records\nimport Cachex.Spec\n\n# create a cache with a ring router\nCachex.start(:my_cache, [\n  router: router(module: Cachex.Router.Ring, options: [\n    monitor: true\n  ])\n])\n```\n\nThis option will listen to `:nodeup` and `:nodedown` events and redistribute keys around your cluster automatically; you will generally always want this enabled if you're planning to dynamically add and remove nodes from your cluster.\n\nYou can also visit the [Cache Routers](./cache-routers.md) documentation for further information on this topic.","ref":"distributed-caches.html#cluster-routers"},{"type":"extras","title":"Distribution Rules - Distributed Caches","doc":"There are a number of behavioural changes when a cache is in distributed state, and it's important to be aware of them. I realise this is a lot of information, but it's good to have it all documented.\n\nCalling other nodes is very simple when it's just retrieving a key, but there are other actions which require more handling. As an example of this, Node A may have 3 keys while Node B may have 2 keys. In this instance, a `Cachex.size/2` call will return a count of `5` automatically by merging the results from both nodes. This is transparent for convenience, and applies to all cache actions. There are cases where you may with to run something like `Cachex.size/2` on a specific node, instead of the whole cluster. For this case, all cache actions also support the `:local` option which, when set to `true`, will return only the result from the local node.\n\nIn the case you're using an action which is based on multiple keys (such as `Cachex.put_many/3` or `Cachex.transaction/3`), all keys within a single call **must** live on the same destination node. This should not be surprising, and is similar to the likes of [Redis](https://redis.io) where this is also the case (at the time of writing). If you attempt to use these types of calls with keys which slot to different nodes, you will receive a `:cross_slot` error. As it's typically difficult to guarantee that your keys will slot to the same node, it's generally recommended to only call these functions with a single key when used in a distributed cache (and so `put_many/3` is then redundant).\n\nThere are a small number of actions which are simply unavailable when called in a distributed environment. An example of this is `Cachex.stream/3`, where there really is no logical approach to a sane implementation. These functions can not be run inside a distributed cache, although you can still opt into running them locally via `:local`.","ref":"distributed-caches.html#distribution-rules"},{"type":"extras","title":"Referencing Functions - Distributed Caches","doc":"There are several actions within Cachex which accept a function as an argument. In these cases it's necessary to provide a reference to a function which is guaranteed to exist on all nodes in a cluster.\n\nTo expand on this, providing an inline function such as `fn x -> x * 2 end` will not work as expected, because it exists only on the local node. If this action is then delegated to a different node, the function no longer exists. Fortunately this is simple to work around, by instead naming the function within a module and providing it via `&MyModule.my_fun/1`.\n\nThis is mainly due ot the naming conventions of anonymous functions, meaning that they cannot be guaranteed to be exactly the same on different OTP nodes. In the case of a named function, even though the anonymous binding is different, it's only passing through to a known function we can guarantee is consistent.\n\nIf this doesn't make sense, just remember to use module functions rather than inlined functions!","ref":"distributed-caches.html#referencing-functions"},{"type":"extras","title":"Locally Available Actions - Distributed Caches","doc":"There are a few cache actions which will always execute locally, regardless of the state of the cache. This is due to either the semantics of their execution, or simply restrictions in their implementation. A good example of this is `Cachex.inspect/3`, which is used to debug the local cache. This wouldn't make sense in a distributed cache, so it doesn't even try.\n\nAnother pair of actions with this limitation are `Cachex.save/3` and `Cachex.restore/3`. Locally running these functions means that all filesystem interaction happens on the local node only, however these functions still provide save/restore functionality in a distributed cache due to how they're written internally.\n\nWhen saving a cache to disk, `Cachex.save/3` makes use of `Cachex.export/2` which *is* available as a distributed action. When restoring a cache from disk, the `Cachex.restore/3` function uses `Cachex.put/4` internally, which is *also* available across nodes. This may be confusing at first, but after much consideration it was determined that this was the most sane design (even if it's quite odd).\n\nIt should also be noted that `Cachex.save/3` supports the `:local` option, and will pass it through to `Cachex.export/2`, making it possible to save only the data on the local node.","ref":"distributed-caches.html#locally-available-actions"},{"type":"extras","title":"Reactive Warming","doc":"# Reactive Warming\n\nWarming a cache reactively is essentially lazily loading a missing key on access. Put another way, Cachex will \"react\" to a missing key by attempting to load it from elsewhere (and then place it in the cache). This is a fairly common need, and lends itself well to a couple of different situations:\n\n* Sporadic calls may result in a saving of resources:\n  * As data is warmed \"on demand\", we only use cache memory when necessary.\n  * There is no wasted cache operation time warming data needlessly.\n* Parameterized calls may result in a short lived window of hot data, such as:\n  * A user session focuses on specific resources often for a brief period of time.\n  * A window of data which is relevant for a brief period of time (i.e. last 60 minutes).\n\nAs data is loaded lazily, this is a very effective approach for data which remains active for only a short period of time. This also means that reactive warmers are very memory efficieny by default, because they load data as it's needed instead of eagerly in anticipation of it being needed.","ref":"reactive-warming.html"},{"type":"extras","title":"Defining a Warmer - Reactive Warming","doc":"To provide this type of warming, Cachex provides the interface function `Cachex.fetch/4`. When calling this action, the developer provides a function containing the code to run in case of a cache miss. The result of this function is used to populate the key in the cache.\n\nThere are several formats you can use to return values from a warming function. The snippet below demonstrates the various recognised return types from a warming function inside `Cachex.fetch/4`.\n\n```elixir\n# start an empty cache\nCachex.start(:cache)\n\n# defining a function alias using shorthand syntax\n{ :commit, 4 } = Cachex.fetch(:cache, \"key1\", &String.length/1)\n{     :ok, 4 } = Cachex.fetch(:cache, \"key1\", &String.length/1)\n\n# defining an inline function using `:commit` syntax\n{ :commit, 4 } = Cachex.fetch(:cache, \"key2\", fn key ->\n  { :commit, String.length(key) }\nend)\n\n# defining an inline function using `:commit` syntax, with options\n{ :commit, 4 } = Cachex.fetch(:cache, \"key3\", fn key ->\n  { :commit, String.length(key), expire: :timer.seconds(60) }\nend)\n\n# define a function which doesn't save the result (i.e. in case of error)\n{ :ignore, 4 } = Cachex.fetch(:cache, \"key4\", fn key ->\n  { :ignore, String.length(key) }\nend)\n```\n\nThere are a few things to point out here explicitly. Firstly, the return value of a call to `Cachex.fetch/4` will contain `:commit` only if the value was loaded by that specific call. If the value already exists in the table, the `:ok` value will be returned instead. As we can see above, we loaded `key1` twice and so the second call received `:ok` as it was already populated.\n\nIn the case of `key3` we're providing options alongside our commit tuple. This is a recent feature which allows us to pass options directly through to `Cachex.put/4` (which `Cachex.fetch/4` uses internally). This means that you're now able to define things like expiration as a function of a lazily loaded value, which is a very flexible model.\n\nFinally the use of `:ignore` in the return tuple allows the developer to opt out of placing the value in the cache. This is useful when handling errors, or cases where data isn't ready for consumption yet. You can still pass a value back to the outer code flow here, it just won't be placed inside the cache.\n\nIn previous versions of Cachex it was possible to store fallback `:state` within a cache (accessible as a second parameter to a fallback function). This has been removed as of v4.x to simplify `Cachex.fetch/4` handling and as it was a lesser used feature. It's possible this feature will be re-added in future if there is enough demand for it.","ref":"reactive-warming.html#defining-a-warmer"},{"type":"extras","title":"Example Use Cases - Reactive Warming","doc":"The use of these warmers allows you to build very simple bindings to reduce overhead on your backing systems. A very common use case is using a `Cachex` instance with reactive warming from a remote system, in order to lower the number of network jumps. With effective use of other cache features such as expiration, you can ensure that your application finds a good balance between avoiding stale data and minimization of network overhead.\n\nAs an example, let's look an application containing an API to retrieve a list of packages in a database. As creation of a package is infrequent, we can avoid calling our remote database every time and instead retrieve a cached value from memory:\n\n```elixir\n# need our records\nimport Cachex.Spec\n\n# initialize our cache with expiration set\nCachex.start_link(:cache, [\n  expiration: expiration(default: :timer.minutes(5))\n])\n\n# retrieve a list of packages to serve via our API\nCachex.fetch(:cache, \"/api/v1/packages\", fn ->\n  { :commit, Repo.all(from p in Package) }\nend)\n```\n\nThe combination of options here (even in this small snippet) means that we'll only call our database **at most** once every 5 minutes. This allows you to easily lower the pressure on backing systems with very little code; a few lines can improve your API performance dramatically!","ref":"reactive-warming.html#example-use-cases"},{"type":"extras","title":"Warmer Contention - Reactive Warming","doc":"One of the most common missteps with warming reactive like this is how an application behaves when a missing key is read concurrently from two places. Cachex makes sure to take these cases into account to guarantee consistency in your application. For a moment, let's forget that `Cachex.fetch/4` exists and instead write a manual example which demonstrates a couple of issues:\n\n```elixir\n# start a new cache\nCachex.start(:cache)\n\n# lazily load a missing value\ncase Cachex.get(:cache, \"key\") do\n  {:ok, nil} ->\n    value = call_database_with_network_delay(\"key\")\n    Cachex.put(:cache, \"key\", value)\n    value\n\n  value ->\n    value\nend\n```\n\nAt a glance this might look fine, but there's one big problem with this approach. As `call_database_with_network_delay/1` takes a long time to run, there's still a period of time in which our key is missing from a cache. This has the nasty side effect that any calls made to this same code between the initial call and the time when `call_database_with_network_delay/1` first returns will spawn additional calls to the database!\n\nFortunately Cachex's design will ensure that only the _first_ warmer executed will fire for a given key, even if more processes ask for the same key before the code completes. The internal Cachex `Courier` service will queue these requests up, and then resolve them all with the result produced by the first. This ensures that you don't have stray processes calling for the same thing (which is especially bad if they're talking to a database, etc.).\n\nYou can think of this as a per-key queue at a high level, with a short circuit involved to avoid executing too often. To see this in action, let's attempt to fetch a key ten times using both our manual approach, as well as using `Cachex.fetch/4`:\n\n```elixir\n# start a new cache\nCachex.start(:cache)\n\n# run manually\nfor _ <- 1..10 do\n  spawn(fn ->\n    case Cachex.get(:cache, \"key1\") do\n      {:ok, nil} ->\n        IO.puts(\"Running warmer after get/2\")\n        value = :timer.sleep(1000)\n        Cachex.put(:cache, \"key1\", value)\n        value\n\n      value ->\n        value\n    end\n  end)\nend\n\n# run via fetch/4\nfor _ <- 1..10 do\n  spawn(fn ->\n    Cachex.fetch(:cache, \"key2\", fn key ->\n      IO.puts(\"Running warmer in fetch/4\")\n      value = :timer.sleep(1000)\n      value\n    end)\n  end)\nend\n```\n\nIf you run this code, you'll see the log of the first loop emit 10 times as each call overlaps and you end up with `:timer.sleep/1` called 10 times. In the second loop you'll only see the log emit a single time, as Cachex knows to queue the subsequent calls to resolve with the result of the first.","ref":"reactive-warming.html#warmer-contention"},{"type":"extras","title":"Proactive Warming","doc":"# Proactive Warming\n\nIntroduced alongside Cachex v3, cache warmers act as an eager way to populate a cache. Rather than waiting for a cache miss to retrieve a value, values will be pulled up front to ensure that there is never a miss. This can be viewed as being _proactive_, whereas `Cachex.fetch/4` can be seen as _reactive_. As such, this is a better tool for those who know what data will be requested, rather than those dealing with arbitrary data.","ref":"proactive-warming.html"},{"type":"extras","title":"Defining a Warmer - Proactive Warming","doc":"To implement this type of warming, Cachex introduced the [Cachex.Warmer](https://hexdocs.pm/cachex/Cachex.Warmer.html) behaviour. This behaviour can be implemented on a module to define the logic you want to run periodically in order to refresh your data from a source. Let's look at defining a very typical proactive warmer, which fetches rows from a database and maps them into a cache table using the `id` field as the cache key:\n\n```elixir\ndefmodule MyProject.DatabaseWarmer do\n  @moduledoc \"\"\"\n  Dummy warmer which caches database rows.\n  \"\"\"\n  use Cachex.Warmer\n\n  @doc \"\"\"\n  Executes this cache warmer with a connection.\n  \"\"\"\n  def execute(connection) do\n    connection\n    |> Database.query\n    |> handle_results\n  end\n\n  # ignores the warmer result in case of error\n  defp handle_results({ :error, _reason }),\n    do: :ignore\n\n  # maps the results into pairs to store\n  defp handle_results({ :ok, rows }) do\n    { :ok, Enum.map(rows, fn(row) ->\n      { row.id, row }\n    end) }\n  end\nend\n```\n\nThis simple warmer will ensure that if you look for a row identifier in your cache, it's always going to be readily available (assuming it exists in the database). The format of the result value must be provided as either `{ :ok, pairs }` or `{ :ok, pairs, options }`. These pairs and options should match the same format you'd use when calling `Cachex.put_many/3`.\n\nTo make use of a warmer, a developer needs to assign it within the `:warmers` option during cache startup. This is where we can also control the frequency with which the warmer is run by setting the `:interval` option (which can also be `nil`):\n\n```elixir\n# for warmer()\nimport Cachex.Spec\n\n# define the cache with our warmer\nCachex.start_link(:cache, [\n  warmers: [\n    warmer(\n      state: connection,\n      module: MyProject.DatabaseWarmer,\n      interval: :timer.seconds(30),\n      required: true\n    )\n  ]\n])\n```\n\nThe `:warmers` option accepts a list of `:warmer` records, which include information about the module, the warmer's state, and various other options. If your cache warmer is necessary for your application, you can flag it as `:required`. This will ensure that your cache supervision tree is not considered \"started\" until your warmer has run successfully at least once.","ref":"proactive-warming.html#defining-a-warmer"},{"type":"extras","title":"Example Use Cases - Proactive Warming","doc":"To demonstrate this in an application, we'll use the same examples from the [Reactive Warming](reactive-warming.md) documentation, which is acting as a cache of an API call to retrieve a list of packages from a database. In the case of a cache miss, reactive warming would call the database and place the result in the cache for future calls.\n\nWith proactive warming, we can go a lot further. As creation of a package is infrequent, we can load the entire list into memory to guarantee we have everything accessible in our cache right from application startup:\n\n```elixir\ndefmodule MyProject.PackageWarmer do\n  @moduledoc \"\"\"\n  Module to warm the packages API.\n  \"\"\"\n  use Cachex.Warmer\n\n  @doc \"\"\"\n  Executes this cache warmer.\n  \"\"\"\n  def execute(_) do\n    # load all of the packages from the database\n    packages = Repo.all(from p in Package)\n\n    # create pairs from the API path and the package\n    package_pairs = Enum.map(packages, fn(package) ->\n      { \"/api/v1/packages/#{package.id}\", package }\n    end)\n\n    # return pairs for the root, as well as all single packages\n    { :ok, [ { \"/api/v1/packages\", packages } | package_pairs ] }\n  end\nend\n```\n\nWe then just provide our warmer during initialization of our cache, and define that it needs to be completed prior to startup via the `:required` flag. The `:interval` option is used to specify that it will refresh every 5 minutes:\n\n```elixir\n# need our records\nimport Cachex.Spec\n\n# initialize our cache\nCachex.start_link(:cache, [\n  warmers: [\n    warmer(\n      module: MyProject.PackageWarmer,\n      interval: :timer.minutes(5),\n      required: true\n    )\n  ]\n])\n```\n\nAs a result of being able to populate many keys at once we have not only populated `\"/api/v1/packages\"` to return the list of packages, but we have also populated the entire API `\"/api/v1/packages/{id}\"`. This is a much more optimized solution for this type of caching, as you can explode out your key writes with a single cache action, while requiring no extra database requests.\n\nSomewhat obviously these warmers can only be used if you know what types of data you're expecting to be cached. If you're dealing with seeded data (i.e. from a user) you probably can't use proactive warming, and should be looking at reactive warming instead. You must also consider how relevant the data is that you're caching; if you only care about it for a short period of time, you likely don't want a warmer as they run for the lifetime of the cache.","ref":"proactive-warming.html#example-use-cases"},{"type":"extras","title":"Triggered Warming - Proactive Warming","doc":"In addition to having your warmers managed by Cachex, it's now also possible to manually warm a cache. As of Cachex v4.x, the interface now includes `Cachex.warm/2` for this purpose. Calling this function will execute all warmers attached to a cache, or a subset of warmers you select at call time:\n\n```elixir\n# warm the cache manually\nCachex.warm(:my_cache)\n\n# warm the cache manually and block until complete\nCachex.warm(:my_cache, wait: true)\n\n# warm the cache manually, but only with specific warmers\nCachex.warm(:my_cache, only: [MyProject.PackageWarmer])\n```\n\nThis is extremely helpful for things like evented cache invalidation and debugging. The Cachex internal management actually delegates through to this under the hood, meaning that there should be no surprising inconsistencies between managed vs. manual warming. It should be noted that `Cachex.warm/2` can be run either with or without an `:interval` set in your warmer record.","ref":"proactive-warming.html#triggered-warming"},{"type":"extras","title":"Custom Commands","doc":"# Custom Commands\n\nCachex allows for custom commands to be attached to a cache, in order to simplify common logic without having to channel all of your cache calls through a specific block of code or a specific module. Cache commands are the solution for extending Cachex with operations or verbiage specific to your application and/or domain without bloating Cachex itself.\n\nCommands operate in such a way that they're marginally quicker than hand-writing your own wrapper functions, but only very slightly. As a rule of thumb you should aim to set only very general actions as commands on a cache, and keep very specific actions outside of the caching layer. It's possible that in future Cachex may ship with some additional built-in commands for very common functionality (perhaps as a separate library).","ref":"custom-commands.html"},{"type":"extras","title":"Defining a Command - Custom Commands","doc":"Commands are defined on a per-cache basis via the `:commands` flag inside the `Cachex.start_link/2` options.\n\nThere are two types of command, either `:read` or `:write`. As you might guess the former will return a modified value from within a cache, while the latter will modify the value inside the cache before returning it.\n\nLet's consider some basic `List` operations, and assume that we're storing some `List` types in a cache. In this case we might wish to have some typical `List` operations attached to our cache, rather than defining them externally.\n\nTwo perfect examples for us to look at are retrieving the last item in a list (`List.last/1`), and also popping the first item from a list (`List.pop_at/3` with index 0). As the former does not need to modify the `List`, it would be classed as a `:read` command. In contrast the latter _does_ need to modify the `List`, and so it would be classed as a `:write` command.\n\nLet's look at how we can define simple versions of these commands and attach them to a cache at startup:\n\n```elixir\n# need the records\nimport Cachex.Spec\n\n# define some custom commands\nlast = &List.last/1\nlpop = fn\n  ([ head | tail ]) ->\n    { head, tail }\n  ([ ] = list) ->\n    {  nil, list }\nend\n\n# attach them to the cache\nCachex.start_link(:my_cache, [\n  commands: [\n    last: command(type:  :read, execute: last),\n    lpop: command(type: :write, execute: lpop)\n  ]\n])\n```\n\nEach command receives a cache value to operate on and return. A command flagged as `:read` (such as `:last` above) will simply transforms the cache value before the final command return occurs, allowing the cache to mask complicated logic from the calling module. Commands flagged as `:write` are a little more complicated, but still fairly easy to grasp. These commands *must* return a 2-element tuple, with the return value in index `0` and the new cache value in index `1`.\n\nIt is important to note that custom cache commands _will_ receive `nil` values in the cache of a missing cache key. If you're using a `:write` command and receive a misisng value, your returned modified value will only be written back to the cache if it's no longer `nil`. This allows the developer to implement logic such as lazy loading, but also escape the situation where you're cornered into writing to the cache.","ref":"custom-commands.html#defining-a-command"},{"type":"extras","title":"Invoking Commands - Custom Commands","doc":"The entry point to command invocation is via the `Cachex.invoke/4` interface function. This function accepts a command name and the key it should be called on. All value retrieval is handled automatically, and errors like invalid command names will result in an error as expected.\n\nLet's look at some examples of calling the new `:last` and `:lpop` commands we defined above, after populating an example list in our cache.\n\n```elixir\n# place a new list into our cache of 3 elements\n{ :ok, true } = Cachex.put(:my_cache, \"my_list\", [ 1, 2, 3 ])\n\n# check the last value in the list stored under \"my_list\"\n{ :ok,    3 } = Cachex.invoke(:my_cache, :last, \"my_list\")\n\n# pop all values from the list stored under \"my_list\"\n{ :ok,    1 } = Cachex.invoke(:my_cache, :lpop, \"my_list\")\n{ :ok,    2 } = Cachex.invoke(:my_cache, :lpop, \"my_list\")\n{ :ok,    3 } = Cachex.invoke(:my_cache, :lpop, \"my_list\")\n{ :ok,  nil } = Cachex.invoke(:my_cache, :lpop, \"my_list\")\n\n# check the last value in the list stored under \"my_list\"\n{ :ok,  nil } = Cachex.invoke(:my_cache, :last, \"my_list\")\n```\n\nWe can see how both commands are doing their job and we're left with an empty list at the end of this snippet. At the time of writing there are no options recognised by `Cachex.invoke/4` even though there _is_ an optional fourth parameter for options, it's simply future proofing.\n\nThis example does highlight one shortcoming that custom commands do have currently; it's not possible to remove an entry from the table inside a custom command yet. This may be supported in future but there's currently no real demand, and adding it would complicate the interface so it's on pause for now.","ref":"custom-commands.html#invoking-commands"},{"type":"extras","title":"Execution Lifecycle","doc":"# Execution Lifecycle\n\nSometimes it might be beneficial to hook into when cache actions occur, and so Cachx provides a way to do so. This is avaiable via the hook system, which allows the developer to specify execution hooks which are notified when actions are carried out. These execution hooks receive messages in the form of tuples, which represent the action taken by the cache and also potentially the result of the action.","ref":"execution-lifecycle.html"},{"type":"extras","title":"Creating a Hook - Execution Lifecycle","doc":"Cachex provides a behaviour `Cachex.Hook` which provides a small abstraction over `GenServer`, with a few tweaks around synchronous execution and argument handling. All cache notifications can be received via `handle_notify/3`, but you still have access to all of the usual `GenServer` callbacks in case you want to add any additional logic to your hook (which is very common).\n\nHooks can become quite complicated, but let's look at a simple example of logging all cache actions to `:stdout` and keeping track of the last cache action executed. As stated above we have access to all the usual `GenServer` callbacks, so we can define a `handle_call/3` callback to retrieve any data we need:\n\n```elixir\ndefmodule MyProject.MyHook do\n  @moduledoc \"\"\"\n  A small hook to log all actions and store the most recent.\n  \"\"\"\n  use Cachex.Hook\n\n  # Initialization.\n  def init(_),\n    do: { :ok, nil }\n\n  # Log the action and result, then store the action.\n  def handle_notify(action, result, _last) do\n    IO.puts(\"Action: #{action}\")\n    IO.puts(\"Result: #{result}\")\n    { :ok, action }\n  end\n\n  # Provide access to the last executed cache action.\n  def handle_call(:last_action, _ctx, last),\n    do: { :reply, last, last }\nend\n```\n\nOnce you have your `Cachex.Hook` definition, you can attach it to a cache a startup using the `:hooks` option on the `Cachex.start_link/2` interface. This option accepts a list of `Cachex.Spec.hook` records and attaches them to the cache on launch:\n\n```elixir\n# need the records\nimport Cachex.Spec\n\n# create a cache with our hook\nCachex.start_link(:my_cache, [\n  hooks: [\n    hook(module: MyProject.MyHook)\n  ]\n])\n```\n\nA minimal hook record will contain just the name of the module implementing the `Cachex.Hook` behaviour, but you can also provide an initial `:state` (defaults to `nil`) and a custom `:name` for the hook process (which defaults to the process identifier). Please see the `Cachex.Hook` documentation for the optional callbacks which can be implemented to configure your hook.","ref":"execution-lifecycle.html#creating-a-hook"},{"type":"extras","title":"Notification Types - Execution Lifecycle","doc":"Each hook notification consists of the name of the cache action being executed, and the list of arguments it was called with. These notifications are of the form `{ action, args }` where `action` is an atom action name and `args` is a list of execution arguments.\n\nBelow is an example just to show this in context of a cache call, assuming we're doing a simple `Cachex.get/3` call:\n\n```elixir\n# given this cache call and result\n{ :ok, \"value\" } = Cachex.get(:my_cache, \"key\")\n\n# you would receive these notification params\n{ :get, [ :my_cache, \"key\" ] }, { :ok, \"value\" }\n```\n\nUsing this pattern makes it simple to hook into specific actions or specific cases (such as error cases), which is a powerful tool enabled by a very simple interface.","ref":"execution-lifecycle.html#notification-types"},{"type":"extras","title":"Provisioning Hooks - Execution Lifecycle","doc":"There are some specific values which cannot be provided to your hook on startup as they have not yet been created. The best and most useful example of this is the cache's inner state, as it allows cache calls without the overhead of looking up the cache location each time.\n\nThe `provisions/0` callback can be used to gain access to such values, by returning a `List` of atoms to signal which items should be provided to your hook. This option will cause your hook to be provided with an instance of what you're asking for, via the `handle_provision/2` callback. An example of this pattern looks like this:\n\n```elixir\ndefmodule MyProject.MyHook do\n  use Cachex.Hook\n\n  # Initialization.\n  def init(_),\n    do: { :ok, nil }\n\n  # Request the cache state as a provision.\n  def provisions,\n    do: [ :cache ]\n\n  # Receive a cache state and store it for later.\n  def handle_provision({ :cache, cache }, _cache),\n    do: { :noreply, cache }\nend\n```\n\nThe message received inside `handle_provision/2` will be of the form `{ type, value }` where `type` is equal to the atom you've requested (in this case `:cache`). Be aware that this modification event may be fired multiple times if the provided value is modified elsewhere, in order to keep hooks in sync with internal changes. Please see `Cachex.Provision` for details on the provisioning behaviour and the available options.","ref":"execution-lifecycle.html#provisioning-hooks"},{"type":"extras","title":"Performance Overhead - Execution Lifecycle","doc":"Although hooks have been fairly well optimized at this point, there is still a minimal overhead to defining a hook.\n\nIf you are using an asynchronous hook, the overhead to the main cache execution flow is only the cost of passing a message to the backing hook process. This is extremely minimal and should pale in comparison to your main application logic, and likely be near irrelevant. In the case of synchronous hooks, we still have this same message passing overhead but of course the logic taken inside the hook itself has an impact on the execution flow.\n\nAs the typical use case for cache hooks is only one or two asynchronous hooks, the notification flow is optimized for this scenario. For this reason it's important to note that hooks are always notified sequentially, as spawning a process per hook would be a dramatic slowdown for asynchronous hoks. This should be kept in mind when using synchronous hooks, as (for example) 5 synchronous hooks each taking 1 second to run would result in a 5 second execution time for a cache call. This isn't necessarily bad if your usage pattern allows for it, but it's something to be aware of.","ref":"execution-lifecycle.html#performance-overhead"},{"type":"extras","title":"Migrating to v4.x","doc":"# Migrating to v4.x\n\nThe release of Cachex v4.x includes a lot of internal cleanup and restructuring. As such, there are quite a few breaking changes to be aware of.\n\nSome of them are simple (like changing names) and others require more involved migration. This page will go through everything and hopefully make it easy for you to upgrade!","ref":"migrating-to-v4.html"},{"type":"extras","title":"Cache Options - Migrating to v4.x","doc":"There are a number of changes to the options provided at cache startup in `Cachex.start_link/2`.\n\nThe `:fallback` option has been removed. This was introduced in earlier versions of Cachex before `Cachex.fetch/4` existed, and it doesn't serve nearly as much purpose anymore. Removing this cleaned up a lot of the internals and removes a lot of hidden magic, so it was time to go. To align with this change, the function parameter of `Cachex.fetch/4` has been changed to be required rather than optional.\n\nBoth the `:stats` and `:limit` options have been removed, in favour of explicitly providing the hooks that back them. These flags were sugar in the past but caused noise and confusion, and it's now much better to have people getting used to using `:hooks`:\n\n```elixir\n# behaviour in Cachex v3\nCachex.start_link(:my_cache, [\n    stats: true,\n    limit: limit(\n        size: 500,\n        policy: Cachex.Policy.LRW,\n        reclaim: 0.5,\n        options: []\n    )\n])\n\n# behaviour in Cachex v4\nCachex.start_link(:my_cache, [\n    hooks: [\n        hook(module: Cachex.Stats),\n        hook(module: Cachex.Limit.Scheduled, args: {\n            500,  # setting cache max size\n            [],   # options for `Cachex.prune/3`\n            []    # options for `Cachex.Limit.Scheduled`\n        })\n    ]\n])\n```\n\nBoth of these features have had additional documentation written, so you can double check the relevant documentation in [Gathering Stats](../management/stats-gathering.md) and [Limiting Caches](../management/limiting-caches.md) as necessary. Limits in particular have had quite a shakeup in Cachex v4, so it's definitely worth a visit to the documentation if you're using those!\n\nIn the example above, you can also see that the `:state` option of a hook definition has been replaced with `:args` in Cachex v4.x. As a hook controls the state during execution, this is to separate it from warmers where `:state` is static.\n\nThe `:nodes` option has also been removed, in favour of the new approach to routing in a distributed cache. It's possible to keep the same semantics as Cachex v3 using the `Cachex.Router.Jump` module implementation:\n\n```elixir\n# behaviour in Cachex v3\nCachex.start_link(:my_cache, [\n    nodes: [\n        :node1,\n        :node2,\n        :node3\n    ]\n])\n\n# behaviour in Cachex v4\nCachex.start_link(:my_cache, [\n    router: router(module: Cachex.Router.Jump, options: [\n        nodes: [\n            :node1,\n            :node2,\n            :node3\n        ]\n    ])\n])\n```\n\nThis is covered in much more detail in the corresponding documentation of [Cache Routers](../routing/cache-routers.md) and [Distributed Caches](../routing/distributed-caches.md); it's heavily recommended you take a look through those pages if you were using `:nodes` in the past.\n\nLast but not least, the `:transactional` flag has been renamed to `:transactions`. Ironically this used to be the name in the Cachex v2 days, but it turned out that it was a mistake to change it in Cachex v3!","ref":"migrating-to-v4.html#cache-options"},{"type":"extras","title":"Warming Changes - Migrating to v4.x","doc":"There are some minor changes to cache warmers in Cachex v4, which require only a couple of minutes to update.\n\nThe `:async` field inside a warmer record has been replaced with the new `:required` field. This is basically equivalent to the inverse of whatever you would have set `:async` to in the past. As cache warmers can now be fired as either async or sync on the fly, this option didn't make much sense anymore. Instead the new `:required` field dictates that a warmer is _required_ to have run before a cache is considered fully started.\n\nThe other change affecting cache warmers is the removal of `interval/0` function from the `Cachex.Warmer` behaviour. The interval is something you might want to change dynamically, and so it didn't make sense to be defined in the code itself. It has been moved to the `:interval` field in the Cachex warmer record, and behaves exactly as before.","ref":"migrating-to-v4.html#warming-changes"},{"type":"extras","title":"Function Parameters - Migrating to v4.x","doc":"There are several naming changes to options passed to functions across the Cachex API. There are no functional differences, so these should be quick cosmetic things to change as needed.\n\nFirst of all the `:ttl` option has been renamed to `:expire` in all places it was supported (mainly `Cachex.put/4` and various wrappers). It was strange to refer to expiration as \"expiration\" all over and have the setting be `:ttl`, so this just makes things more consistent.\n\nThe `:initial` option for `Cachex.incr/4` and `Cachex.decr/4` has been renamed to `:default`. This makes way more sense and is much more intuitive; it was probably just a misnaming all those years ago that stuck. Time to make it better!\n\nFor all of the functions which support `:batch_size`, namely `Cachex.stream/3` and functions which use it, this has now been renamed to `:buffer`. The previous name was too close to the underlying implementation, whereas the new name is much more generic (and shorter to type!).","ref":"migrating-to-v4.html#function-parameters"},{"type":"extras","title":"Removed & Renamed APIs - Migrating to v4.x","doc":"There are several changes to the main Cachex API, including removal of some functions and naming changes of others.\n\nThe `count/2` function has been removed in favour of `Cachex.size/2`. These two functions did almost the same thing, the only difference was that `Cachex.size/2` would return the size of the cache including unpurged expired records, while `count/2` would filter them out. Instead of two functions for this, you can now opt into this via `Cachex.size/2`:\n\n```elixir\n# total cache entry count\nCachex.size(:my_cache)\nCachex.size(:my_cache, expired: true)\n\n# ignores expired but unremoved entries\nCachex.size(:my_cache, expired: false)\n```\n\nThis should hopefully feel more intuitive, while allowing us to trim a bunch of the Cachex internals. The underlying implementations are identical, so it should be easy to migrate if you need to.\n\nBoth functions `dump/3` and `load/3` have been renamed in Cachex v4. These names were terrible to begin with, so it's about time they're changed! Instead we now have `Cachex.save/3` and `Cachex.restore/3`, which behave in exactly the same way (aside from being a bit cleaner in implementation!).\n\nFinally the two deprecated functions `set/4` and `set_many/3` have finally been removed. If you were using these, please use `Cachex.put/4` and `Cachex.put_many/3` instead from now on.","ref":"migrating-to-v4.html#removed-renamed-apis"},{"type":"extras","title":"Other Miscellaneous Changes - Migrating to v4.x","doc":"There are a few other small changes which don't really need much explanation, but do need to be noted for reference.\n\nThe minimum supported Elixir version has been raised from Elixir 1.5 to Elixir 1.7. In reality there are probably very few people out there still using Elixir 1.7 and it could be raised further, but there's also nothing really compelling enough to make this happen at this time.\n\nA lot of the record types in Cachex v4 had their orders changed, so if anyone was matching directly (instead of using record syntax) they should adapt to using `entry(entry, :field)` instead.\n\nThe former `ExecutionError` has been replaced with `Cachex.Error`, which is a combination of several smaller modules. This is just a naming difference to hopefully make it easier to type and remember!","ref":"migrating-to-v4.html#other-miscellaneous-changes"},{"type":"extras","title":"Migrating to v3.x","doc":"# Migrating to v3.x\n\nThere are many breaking changes bundled up in v3, and so this guide serves as a quick overview of what you'll probably need to check out during adoption. This won't focus on every change, just the user facing ones that can affect how your application code interacts with a cache. It also doesn't serve as a guide on how to change things, just pointing out on which areas the new documentation should be consulted on.","ref":"migrating-to-v3.html"},{"type":"extras","title":"Startup Options - Migrating to v3.x","doc":"In order to reduce a lot of the option parsing involved in Cachex, the options given to `start_link/1` and `start/1` have changed pretty drastically. The easiest way to see the differences is to look again at the documentation for these functions, but here's a very high level summary of changes (if these are things you use, please do check the docs):\n\n- The `:commands` option now expects a list of `command` records.\n- The `:default_ttl`, `:disable_ode` and `:ttl_interval` are now passed as an `expiration` record in the `:expiration` option (and have had their names changed).\n- The `:ets_opts` option has been completely removed.\n- The `:fallback` option now expects a `fallback` record.\n- The `:hooks` option now expects a list of `hook` records.\n- The `:limit` option now expect a `limit` record rather than a `%Cachex.Limit{}` (the shorthand of an integer is still valid).\n- The `:record_stats` option has had the name changed simply to `:stats`.\n- The `:transactions` option has also been renamed to `:transactional`.\n\nAll of these changes are based around improvements to the internal cache states and should also make it easier to understand the structures being passed around, whereas previously there were a lot of loose Keyword definitions. Please see either the module documentation, or guides for the feature you're using, for examples on how to use the new options.","ref":"migrating-to-v3.html#startup-options"},{"type":"extras","title":"Fallbacks - Migrating to v3.x","doc":"Fallback caching has changed quite significantly in v3; there is no longer a `:fallback` option on calls to `get/3`, or any other calls which previously supported it. There are a few reasons for this, but the general one being that it was not always intuitive which calls did in fact support fallbacks. People consistently requested a feature that already existed, which means that it needed to be made more obvious. Due to this there will be a new `fetch/4` function in the main interface which replaces the behaviour. Going forward, the root term \"fetch\" will be used as synonymous to \"fallback\".\n\nHere is an example of the previous v2.x branch vs. the same behaviour in the v3.x branch:\n\n```elixir\n# v2.x using the `:fallback` option to `get/3`.\nCachex.get(:my_cache, \"key\", fallback: &String.reverse/1)\n\n# v3.x using the `fetch/4` signature.\nCachex.fetch(:my_cache, \"key\", &String.reverse/1)\n```\n\nThe signature change allows for an explicit function in the API dedicated to lazy evaluation, and should hopefully be more approachable to those new to the project. It should be noted that the function being passed to `fetch/4` *is* optional if you have set a default fallback function in the main cache options.","ref":"migrating-to-v3.html#fallbacks"},{"type":"extras","title":"Hooks - Migrating to v3.x","doc":"","ref":"migrating-to-v3.html#hooks"},{"type":"extras","title":"Interface - Migrating to v3.x","doc":"Hooks have changed pretty drastically, and yet they should be pretty close to what already exists. The main difference as of Cachex v3 is that Hooks are now driven by the behaviour a lot more than previously. When registering a hook on a cache, you now provide a `hook` record rather than a struct. These records consist of purely the hook module, the hook state, and an optional name to use to register the hook with. Everything else is now driven by behaviour functions in the module registered. This decision was taken as hooks remain fairly constant for a specific job, and so moving into the module definition makes a lot of sense.\n\nRather than define the changes here, please see the documentation for Hooks going forward. Generally the options from the old `%Cachex.Hook{}` struct have moved to have analogous functions in the module behaviour, but please check to be certain.","ref":"migrating-to-v3.html#interface"},{"type":"extras","title":"Provisions - Migrating to v3.x","doc":"Nothing much to say here other than the previous `:worker` provision has been renamed over to `:cache` as the notion of cache workers is now redundant (and has been for a long time at this point). You can still use the received provision in the same way, it's just a tag change to make it clearer as to what is being delivered.","ref":"migrating-to-v3.html#provisions"},{"type":"extras","title":"Streams - Migrating to v3.x","doc":"Cache streams no longer accept the `:of` option, as it was oversimplifying exactly what was happening (and thus prone to error). However, you can now pass an ETS match specification as the second argument to filter internally in ETS before entering the stream. This is a small optimization, but also makes the developer think about matching more, rather than not appreciating exactly what was happening.\n\nA happy side effect of this is that a stream will now respect the expiration time of records (at stream creation time), whereas before you could still receive expired records in the stream output (and would, regularly).","ref":"migrating-to-v3.html#streams"},{"type":"extras","title":"Miscellaneous - Migrating to v3.x","doc":"","ref":"migrating-to-v3.html#miscellaneous"},{"type":"extras","title":"Automatic Janitor - Migrating to v3.x","doc":"The Janitor is enabled by default as of Cachex v3. It seems that explicitly turning it off is preferable to explicitly turning it on for the developer experience, as you tend to assume it's just running by default - and then you panic when nothing is being removed. Minor change, but technically incompatible so worth mentioning.","ref":"migrating-to-v3.html#automatic-janitor"},{"type":"extras","title":"Incr & Decr - Migrating to v3.x","doc":"Previously the `:amount` option dictated how much the value should be incremented/decremented by, but at this point it's an extra argument (the third parameter), which will default to `1`.","ref":"migrating-to-v3.html#incr-decr"},{"type":"extras","title":"Set vs. Put - Migrating to v3.x","doc":"This is a minor change, but worth mentioning. Going forward `set/4` has been replaced with `put/4`. This is nothing more than a name change, as @fishcakez rightfully pointed out that `put/4` is a better naming convention for Elixir. The old `set/4` has been deprecated and simply forwards to `put/4`, so you should likely migrate to avoid that extra function hop :).","ref":"migrating-to-v3.html#set-vs-put"},{"type":"extras","title":"Statistics - Migrating to v3.x","doc":"The format of the map being returned from `Cachex.stats/2` has been modified due to some normalization which took place; this will look much clearer and adopts snake_case over camelCase (etc). It also correctly tracks custom invocations at this point, rather than ignoring them (like it did previously).","ref":"migrating-to-v3.html#statistics"},{"type":"extras","title":"Missing Values - Migrating to v3.x","doc":"In earlier versions of Cachex, `{ :missing, nil }` would be returned to signal that a value did not exist in the cache. This has been removed to simply return `{ :ok, nil }` because (believe it or not) the overhead of figuring out if something was missing was actually quite large in some cases. If you need the same behaviour, you should avoid setting `nil` explicitly in your cache and put something else in instead - that way `{ :ok, nil }` is semantically the same as `{ :missing, nil }`.","ref":"migrating-to-v3.html#missing-values"},{"type":"extras","title":"Migrating to v2.x","doc":"# Migrating to v2.x\n\nIf anything is not covered in here, or there are any issues with anything written in here, please file an issue and I'll get it taken care of.","ref":"migrating-to-v2.html"},{"type":"extras","title":"Distribution - Migrating to v2.x","doc":"We'll start with the big one;\n\nIn the v1.x line of Cachex, there was a notion of remote Cachex instances which have been removed in v2.x onwards. This is a design decision due to the limitations of supporting remote instances and the complexities involved, specifically with regards to discovery and eviction policies.\n\nIn order to migrate away from this, you should now implement a backing datastore such as Redis or Memcached as your master copy and make use of the Cachex fallback behaviour to replicate the data to your local nodes. To handle the removal of data from remote nodes, you should set a TTL on your data and it will periodically and flush automatically. This should support most cases that people were using the distributed nature of Cachex for, but with the main difference that the consistency is now guaranteed and will remain eventually consistent on the local nodes.\n\nThe decision to remove the remote interface does not come lightly; I have spent many weeks trying to conjure something which satisfies the desire of both speed and distribution and the sad truth is that it's quite simply hard to do well. The consistency issues which plague the land of distributed data are just not possible to handle whilst keeping Cachex as fast as it is (and at the end of the day, a cache is supposed to be fast). The final tipping point was the concept of building LRU style caches in a remote context; it's simply not possible to guarantee the consistency of your data without a huge performance hit (we're talking upwards of 1000x slower) due to Cachex operating in the realm of a microsecond.\n\nDo not despair though; if you were totally set on using a native Elixir/Erlang datastore without having to have something separate such as Redis, I'm planning on writing a separate library which is dedicated more to handling the distributed nature as opposed to the feature set that Cachex offers. At the end of the day, I see caching as a different use case to remote data replication - I believe remote Cachex was closer to a distributed state table, rather than a local mirror of data.\n\nIn addition, you can obviously keep on using Cachex `v1.x` as long as you need - it's still on Hex.pm and has a tag on the repo. I can't promise anything new will be added to that codebase, but for what it's worth I do intend to answer any issues reporting bugs on that branch, so file issues as you see fit - just make sure to flag that you're talking about `v1.x`.","ref":"migrating-to-v2.html#distribution"},{"type":"extras","title":"Fallbacks - Migrating to v2.x","doc":"The options and interface for fallback functions have changed a little bit in order to optimize their efficiency and just remove some bloat from the fallback flow.\n\nIn the v1.x branch of Cachex, there were two cache options related to fallbacks; `:default_fallback` and `:fallback_args`. This was a little clumsy looking, and so this has been unified in v2.x to only be a simple `:fallback` option. This can either be a function, or list of fallback options. Below are some examples:\n\n```elixir\n# fallback with no state\n[ fallback: fn(key) -> do_fallback(key) end ]\n[ fallback: [ action: fn(key) -> do_fallback(key) end ] ]\n\n# fallback with a state\n[\n  fallback: [\n    state: db_client,\n    action: fn(key, client) ->\n      retrieve_from_db(client, key)\n    end\n  ]\n]\n\n# provide a state but no default fallback\n[ fallback: [ state: db_client ] ]\n```\n\nIt should be noted that the `state` is passed in as a second argument in the case that the `state` provided is not `nil`. This is another change to previously where you would provide a list and have arbitrarily long arguments. This change was made as it's a more efficient way of calling a fallback and lessens the overhead involved.","ref":"migrating-to-v2.html#fallbacks"},{"type":"extras","title":"Hook Interface - Migrating to v2.x","doc":"Hooks have undergone a bit of tweaking in v2 simply because they were built back when I wasn't fully familiar with the `Gen*` models. The changes are easy to adopt and shouldn't take you much more than a few minutes to modify your codebase:","ref":"migrating-to-v2.html#hook-interface"},{"type":"extras","title":"Callbacks - Migrating to v2.x","doc":"The biggest change made to Hooks is that we have migrated from `GenEvent` to `GenServer`. This means that if you're implementing Hooks, you need to respect the return formats of the `GenServer` module rather than that of `GenEvent`. This only affects the `Gen*` callbacks, such as `handle_call/2` and `handle_info/2`, so if you haven't used them you're going to be fine in this respect.\n\nThe `handle_call/2` callback should become `handle_call/3`, with a new second parameter which is simply the context of the call (and you likely won't ever use). In addition, the return type now becomes `{ :reply, reply, new_state }` instead of `{ :ok, reply, new_state }` - so just a few characters to tweak there. The same applies for `handle_info/2` in that you need to change `{ :ok, new_state }` to `{ :noreply, new_state }`.\n\nThe reason the change was made is that your hooks now live in the Supervision tree alongside the cache, rather than under a `GenEvent` process. This allows shutdown to run more smoothly, and just generally lays out the tree much better. In addition, you now gain access to `handle_cast/2` from the `GenServer` module and it's a much more familiar interface to deal with as opposed to `GenEvent`, which is falling more and more out of use by most Elixir developers.","ref":"migrating-to-v2.html#callbacks"},{"type":"extras","title":"Defaults - Migrating to v2.x","doc":"The only big change here is that the `:type` option of a hook previously defaulted to being a `:pre` hook. This has now changed to default to a `:post` hook.\n\nThe reasoning behind this is that post hooks are a more common use case - you typically don't want to react to the desire to do something, you want to react to something happening. It was also quite easy to become confused when trying to play with results and receiving nothing. This sucked, because it meant that an entirely different function would be called because of the arity changes when requesting results.\n\nThis is a very easy thing to update, and you can always make sure to specify `:type` on your Hooks in future to avoid relying on this default (I imagine most people have done that anyway, so good job!).","ref":"migrating-to-v2.html#defaults"},{"type":"extras","title":"Message Format - Migrating to v2.x","doc":"After talking to a couple of people on the Slack channels, it dawned on me that the current Hook message implementation is quite bad - in the sense that there's a performance hit, and it's awkward to use. The currently pattern behaves as a Tuple of action arguments, so `Cachex.get(:cache, \"key\", opts)` would forward as `{ :get, \"key\", opts }`.\n\nAt a glance it looks like there's nothing wrong with this, but it makes pattern matching difficult and there's clearly a Tuple construction to create that message. Going forward, it is now guaranteed that a message will be a two-element Tuple, with the action as a tag and a list of arguments - so the above would become `{ :get, [ \"key\", opts ]}`.\n\nThis change makes it super easy to pattern match on the action name (for example, the new LRW hook only activates on write actions), and gives you the guarantee that your message will always be the same form as opposed to having arbitrarily long Tuples (which is pretty much always the wrong thing to do).","ref":"migrating-to-v2.html#message-format"},{"type":"extras","title":"Results - Migrating to v2.x","doc":"The decision has been made to always provide results to a post hook, in order to keep the backing logic simple and remove some conditions. It's cheap to forward the results, so there's no real overhead to doing this. Previously the intent was to separate the concerns, but it just led to confusing message handling due to having to use both `handle_notify/2` and `handle_notify/3`. Going forward, you will only ever use `handle_notify/3` (because `handle_notify/2` has been removed). This means that results are also given to your `:pre` hooks, but they're **always** `nil` inside a pre hook and can just be ignored. You should note that if you were previously using `results: true` in your Hook, you shouldn't need to change anything. Examples below:\n\n```elixir\n# old format\ndef handle_notify(msg, state)\n\n# new format\ndef handle_notify(msg, result, state)\n```\n\nObviously because this is always enabled there's no need for an option, and so the `:results` option has been removed from the Hook struct - so you need to drop it from any Hook definitions you have.","ref":"migrating-to-v2.html#results"},{"type":"extras","title":"Options - Migrating to v2.x","doc":"There are a few minor tweaks to the options when starting a cache:\n\n1. The first change is that the previously deprecated `:name` option has been removed. You should now use `start_link/3` or `start/3` and pass the cache name as the first argument. This is to remove some complexity with name validation (in that it's easier to pick out now without parsing options first).\n\n2. The `:ttl_interval` could previously be disabled if set to `false`. This has changed as it's required to be numeric at this point for other reasons, so going forward you should pass `-1` if you wish to disable the interval.\n\n3. This is a small change, but it bothered me often enough to make it. The `:default_fallback` option has been simply renamed to `:fallback`, as it's much easier to write and it's more consistent with the same option inside a `get/3` call (which is also `:fallback`). Long gone are the days in which you pass `:fallback` to a cache only to have it ignored.","ref":"migrating-to-v2.html#options"},{"type":"extras","title":"Transactions - Migrating to v2.x","doc":"As of Cachex v2.x, Mnesia has been removed in favour of direct ETS integration. As a result of this, there are several changes in the way transactions work.\n\nThe first change is down to optimizations of key locking, and requires that you now pass a list of keys to lock as your second parameter to a `transaction/3` call. This is part of the new locking implementation which allows for several optimizations by being explicit with your locks. This optimization provides roughly a 5x speedup, so it's much more efficient than previously. This is pretty easy to adopt:\n\n```elixir\nCachex.transaction(:my_cache, [ \"key1\" ], fn(state) ->\n  old_val = Cachex.get!(\"key1\")\n  new_val = do_something(old_val)\n  Cachex.put!(\"key1\", new_val)\nend)\n```\n\nIf you write to a key which has not been defined in the `keys` parameter, please be aware that it will not be locked and may be written by other processes during your transaction. It also goes without saying that nested transactions should only operate on a subset of keys in an outer transaction.\n\nThe second change is that there is no longer support for `abort/1` from within a transaction, meaning that all writes happen immediately even within your transaction. I don't believe this should be difficult to adopt, as I would imagine that `abort/1` is only used infrequently. It should not be hard to simply rework your transaction flow to exit as needed.\n\nThe final thing to note here is that transactions are all handled by a lock process, which means you should try to avoid causing a bottleneck in your transactions. For example, if you need to check list membership or create new Tuples, try do this outside your transaction first and simply pass it through - this will lessen the time spent in the transaction process and improve performance with transactions. This isn't always possible, but try to optimize like this when applicable.","ref":"migrating-to-v2.html#transactions"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}